//
// File bmptopic.cpp
// Functions for converting a Windows device-independent bitmap to a Psion picture
//
// Written by LCR, January 1995
// (C) Copyright Psion PLC 1995
//

#define WINDOWS_LEAN_AND_MEAN
#include <windows.h>
#include "psiwinxl.h"

typedef struct
	{
	char app_id[3];
	BYTE app_id_chksum;
	BYTE file_ver;
	BYTE app_ver;
	WORD number_of_bitmaps;
	} P_FSIG;

typedef struct
	{
	WORD crc_bits_checksum;
	WORD x_pels;
	WORD y_pels;
	WORD bitmap_bytes;
	LONG offset;
	} WS_PIC_HEADER;

static POINT BmpDims;
static P_FSIG p_fsig;
static WS_PIC_HEADER ws_pic_header1,ws_pic_header2;

WORD p_crctab[256] =
	{
	0x0000,0x1021,0x2042,0x3063,0x4084,0x50A5,0x60C6,0x70E7,0x8108,0x9129,0xA14A,
	0xB16B,0xC18C,0xD1AD,0xE1CE,0xF1EF,0x1231,0x0210,0x3273,0x2252,0x52B5,0x4294,
	0x72F7,0x62D6,0x9339,0x8318,0xB37B,0xA35A,0xD3BD,0xC39C,0xF3FF,0xE3DE,0x2462,
	0x3443,0x0420,0x1401,0x64E6,0x74C7,0x44A4,0x5485,0xA56A,0xB54B,0x8528,0x9509,
	0xE5EE,0xF5CF,0xC5AC,0xD58D,0x3653,0x2672,0x1611,0x0630,0x76D7,0x66F6,0x5695,
	0x46B4,0xB75B,0xA77A,0x9719,0x8738,0xF7DF,0xE7FE,0xD79D,0xC7BC,0x48C4,0x58E5,
	0x6886,0x78A7,0x0840,0x1861,0x2802,0x3823,0xC9CC,0xD9ED,0xE98E,0xF9AF,0x8948,
	0x9969,0xA90A,0xB92B,0x5AF5,0x4AD4,0x7AB7,0x6A96,0x1A71,0x0A50,0x3A33,0x2A12,
	0xDBFD,0xCBDC,0xFBBF,0xEB9E,0x9B79,0x8B58,0xBB3B,0xAB1A,0x6CA6,0x7C87,0x4CE4,
	0x5CC5,0x2C22,0x3C03,0x0C60,0x1C41,0xEDAE,0xFD8F,0xCDEC,0xDDCD,0xAD2A,0xBD0B,
	0x8D68,0x9D49,0x7E97,0x6EB6,0x5ED5,0x4EF4,0x3E13,0x2E32,0x1E51,0x0E70,0xFF9F,
	0xEFBE,0xDFDD,0xCFFC,0xBF1B,0xAF3A,0x9F59,0x8F78,0x9188,0x81A9,0xB1CA,0xA1EB,
	0xD10C,0xC12D,0xF14E,0xE16F,0x1080,0x00A1,0x30C2,0x20E3,0x5004,0x4025,0x7046,
	0x6067,0x83B9,0x9398,0xA3FB,0xB3DA,0xC33D,0xD31C,0xE37F,0xF35E,0x02B1,0x1290,
	0x22F3,0x32D2,0x4235,0x5214,0x6277,0x7256,0xB5EA,0xA5CB,0x95A8,0x8589,0xF56E,
	0xE54F,0xD52C,0xC50D,0x34E2,0x24C3,0x14A0,0x0481,0x7466,0x6447,0x5424,0x4405,
	0xA7DB,0xB7FA,0x8799,0x97B8,0xE75F,0xF77E,0xC71D,0xD73C,0x26D3,0x36F2,0x0691,
	0x16B0,0x6657,0x7676,0x4615,0x5634,0xD94C,0xC96D,0xF90E,0xE92F,0x99C8,0x89E9,
	0xB98A,0xA9AB,0x5844,0x4865,0x7806,0x6827,0x18C0,0x08E1,0x3882,0x28A3,0xCB7D,
	0xDB5C,0xEB3F,0xFB1E,0x8BF9,0x9BD8,0xABBB,0xBB9A,0x4A75,0x5A54,0x6A37,0x7A16,
	0x0AF1,0x1AD0,0x2AB3,0x3A92,0xFD2E,0xED0F,0xDD6C,0xCD4D,0xBDAA,0xAD8B,0x9DE8,
	0x8DC9,0x7C26,0x6C07,0x5C64,0x4C45,0x3CA2,0x2C83,0x1CE0,0x0CC1,0xEF1F,0xFF3E,
	0xCF5D,0xDF7C,0xAF9B,0xBFBA,0x8FD9,0x9FF8,0x6E17,0x7E36,0x4E55,0x5E74,0x2E93,
	0x3EB2,0x0ED1,0x1EF0
	};

DWORD GetDibInfoHeaderSize(void __far *lpDib)
	{
	return(((LPBITMAPINFOHEADER)lpDib)->biSize);
	}

WORD GetDibWidth(HBITMAP hDib)
	{                     
	WORD	DibWidth;

	LPVOID lpDib=GlobalLock(hDib);
	 DibWidth=(WORD)(((LPBITMAPINFOHEADER)lpDib)->biWidth);
	GlobalUnlock(hDib);
	return(DibWidth);
	}

WORD GetDibHeight(HBITMAP hDib)
	{
	LPVOID	lpDib;
	WORD	DibHeight;

	lpDib=GlobalLock(hDib);
	DibHeight=(WORD)(((LPBITMAPINFOHEADER)lpDib)->biHeight);
	GlobalUnlock(hDib);
	return(DibHeight);
	}

BYTE __huge *GetDibBitsAddr(BYTE __huge *lpDib)
	{
	DWORD	dwNumColors, dwColorTableSize;
	WORD	wBitCount;

	wBitCount = ((LPBITMAPINFOHEADER)lpDib)->biBitCount;

	if(GetDibInfoHeaderSize(lpDib) >= 36)
		dwNumColors = ((LPBITMAPINFOHEADER)lpDib)->biClrUsed;
	else
		dwNumColors = 0;

	if (dwNumColors == 0)
		{
		if(wBitCount != 24)
			dwNumColors = 1L << wBitCount;
		else
			dwNumColors = 0;
		}
	dwColorTableSize = dwNumColors * sizeof(RGBQUAD);
	return(lpDib + GetDibInfoHeaderSize (lpDib) + dwColorTableSize);
	}

WORD DibNumColors (LPVOID pv)
	{
	int					bits;
	LPBITMAPINFOHEADER	lpbi;

	lpbi = ((LPBITMAPINFOHEADER)pv);
    if (lpbi->biClrUsed != 0)
		return (WORD)lpbi->biClrUsed;
	bits = lpbi->biBitCount;
	switch (bits)
		{
	    case 1:		return 2;
	    case 4:		return 16;
	    case 8:		return 256;
	    default:	return 0;
		}
	}

HBITMAP LoadDib (LPCSTR szFileName,LPINT errorstate)
	{
	DWORD				dwDibSize,dwHeaderSize,numbytes;
	BYTE __huge			*lpDib;
	HFILE				hFile;
	HBITMAP				hDib;
	BITMAPFILEHEADER	bmfh;

	if((hFile=_lopen(szFileName,OF_READ))==HFILE_ERROR)
		{
        *errorstate=1;
		return(NULL);
		}

	if (HFILE_ERROR==(numbytes=_hread(hFile,&bmfh,sizeof(BITMAPFILEHEADER))))
		{
        *errorstate=2;
		_lclose(hFile);
		return(NULL);
		}

	if (bmfh.bfType != *(LPWORD)"BM")
		{
        *errorstate=10;
		_lclose(hFile);
		return(NULL);
		}

	dwDibSize = bmfh.bfSize - (DWORD)sizeof(BITMAPFILEHEADER);
	if(dwDibSize>65535) /* Avoid memory problems by ignoring files greater than 64k */
		{
        *errorstate=11;
		_lclose(hFile);
		return(NULL);
		}
	hDib=GlobalAlloc(GMEM_MOVEABLE, dwDibSize);
	if (hDib == NULL)
		{
        *errorstate=3;
		_lclose(hFile);
		return(NULL);
		}
	lpDib=(unsigned char __huge *)GlobalLock(hDib);

	if(HFILE_ERROR==(numbytes=_hread(hFile,lpDib,(LONG)dwDibSize)))
		{
        *errorstate=2;
		_lclose(hFile);
		GlobalUnlock(hDib);
		GlobalFree(hDib);
		return(NULL);
		}

	_lclose(hFile);

	dwHeaderSize = GetDibInfoHeaderSize(lpDib);
	if (dwHeaderSize < 12 || (dwHeaderSize > 12 && dwHeaderSize < 16))
		{
        *errorstate=10;
		GlobalUnlock(hDib);
		GlobalFree(hDib);
		return(NULL);
		}          
	if((((LPBITMAPINFOHEADER)lpDib)->biCompression)!=BI_RGB)
		{
        *errorstate=12;
		GlobalUnlock(hDib);
		GlobalFree(hDib);
		return(NULL);
		}          
	if(((((LPBITMAPINFOHEADER)lpDib)->biBitCount)!=4) && ((((LPBITMAPINFOHEADER)lpDib)->biBitCount)!=8))
		{
        *errorstate=12;
		GlobalUnlock(hDib);
		GlobalFree(hDib);
		return(NULL);
		}          
	BmpDims.x=GetDibWidth(hDib);
	BmpDims.y=GetDibHeight(hDib);
	LONG xbits=BmpDims.x;
	LONG ybits=BmpDims.y;
	if((xbits/4*ybits)>65535)
		{
        *errorstate=11;
		GlobalUnlock(hDib);
		GlobalFree(hDib);
		return(NULL);
		}          
	GlobalUnlock(hDib);
	return(hDib);
	}

VOID CalcCrcChksum(WORD __huge *pcrc,BYTE __huge *buf,int cnt)
	{
	while (cnt--)
	*pcrc=((*pcrc)<<8)^p_crctab[((*pcrc)>>8)^(*buf++)];
	}


VOID InitPicFileHeader(LONG nBmpBits,POINT Dim)
	{
	p_fsig.app_id[0]				='P';
	p_fsig.app_id[1]				='I';
	p_fsig.app_id[2]				='C';						
	p_fsig.app_id_chksum			='P'+'I'+'C';
	p_fsig.file_ver					=0x30;
	p_fsig.app_ver					=0x30;
	p_fsig.number_of_bitmaps		=2;

	ws_pic_header1.crc_bits_checksum=0x00;
	ws_pic_header1.x_pels			=(WORD)Dim.x;
	ws_pic_header1.y_pels			=(WORD)Dim.y;
	ws_pic_header1.bitmap_bytes		=(WORD)(nBmpBits/2);
	ws_pic_header1.offset			=0xc;

	ws_pic_header2.crc_bits_checksum=0x00;
	ws_pic_header2.x_pels			=(WORD)Dim.x;
	ws_pic_header2.y_pels			=(WORD)Dim.y;
	ws_pic_header2.bitmap_bytes		=(WORD)(nBmpBits/2);
	ws_pic_header2.offset			=nBmpBits/2;
	}

BYTE SetMonochrome(DWORD RGB,int x,int y)
	{				
	BYTE PelColRef;
	if(RGB<=153)
		PelColRef=0;
	else if(RGB<=306)
		PelColRef=1;
	else if(RGB<=459)
		PelColRef=2;
	else if(RGB<=612)
		PelColRef=3;
	else
		PelColRef=4;
	if(PelColRef==1 && x%2==y%2)
		PelColRef=0;
	if(PelColRef==3 && x%2==y%2)
		PelColRef=4;
	if((PelColRef==1 || PelColRef==3) && x%2!=y%2)
		PelColRef=2;
	return(PelColRef);
	}                       
	
BYTE GetPixelCol(BITMAPINFOHEADER *lpDib,BYTE __huge *lpDibBits,int x,int y)
	{              
	BYTE	index;
	DWORD	col,longwth,offset;
	RGBQUAD	colour;            
	RGBQUAD	*coltab=(RGBQUAD *)(((LPBYTE)lpDib)+((BYTE)lpDib->biSize));
	
	if(lpDib->biCompression!=BI_RGB)
		return(0);              
	y=(int)lpDib->biHeight-y-1;
	switch(lpDib->biBitCount)
		{
		case 1:    
			longwth=((( (((int)(lpDib->biWidth)+7)/8) +3)/4)*4);
			offset=longwth*y+x/8;
			index=*(lpDibBits+offset);
			index>>=(7-x);
			index&=0x01;
			colour=coltab[index];
			col=(WORD)colour.rgbRed+(WORD)colour.rgbGreen+(WORD)colour.rgbBlue;
			break;
		case 4:   
			longwth=((( (((int)(lpDib->biWidth)+1)/2) +3)/4)*4);
			offset=longwth*y+x/2;
			index=*(lpDibBits+offset);
			if(x%2==1)
				index&=0x0f;
			else
				{
				index>>=4;
				index&=0x0f;
				}
			colour=coltab[index];
			col=(WORD)colour.rgbRed+(WORD)colour.rgbGreen+(WORD)colour.rgbBlue;
			break;
		case 8:   
			longwth=((((int)(lpDib->biWidth)+3)/4)*4);
			offset=longwth*y+x;
			index=*(lpDibBits+offset);
			colour=coltab[index];
			col=(WORD)colour.rgbRed+(WORD)colour.rgbGreen+(WORD)colour.rgbBlue;
			break;
		case 24:  
			longwth=((( ((int)(lpDib->biWidth)*3) +3)/4)*4);
			offset=longwth*y+x*3;
			col=(WORD)*(lpDibBits+offset)+(WORD)*(lpDibBits+offset+1)+(WORD)*(lpDibBits+offset+2);
			break;
		}
	
	
	return(SetMonochrome(col,x,y));
	
	}

HGLOBAL ExtractBmpBits(const HBITMAP hBmp,LPLONG nBmpBits,XMSTATPROC statProc)
	{
	int			pct;                                    
	long		i,j,k;
	BYTE		PokePels,TempPels,PelCol;
	BYTE __huge	*lpDib;
	BYTE __huge	*lpDibBits;
	BYTE __huge	*pBmpBytes;
	BYTE __huge	*pTempBmpBytes;
	DWORD		MemSize;
	HGLOBAL		hGlobal;           

	if (BmpDims.x==0||BmpDims.y==0)
		return(NULL);
	MemSize=((BmpDims.x+15)/16)*BmpDims.y*4;

	if (!(hGlobal=GlobalAlloc(GMEM_ZEROINIT,MemSize)))
		{
		return(NULL);
		}

	if (!(pTempBmpBytes=pBmpBytes=(unsigned char __huge *)GlobalLock(hGlobal)))
		{
		return(NULL);
		}                  
	lpDib=(unsigned char __huge *)GlobalLock(hBmp);
	lpDibBits=GetDibBitsAddr(lpDib);

	for(j=0;j<BmpDims.y;j++)
		{
		for(i=0;i<BmpDims.x;i+=8)
			{
			TempPels=0x01;
			PokePels=0;
			for(k=0;k<8;k++)
				{
				PelCol=GetPixelCol((BITMAPINFOHEADER *)lpDib,lpDibBits,(int)(i+k),(int)j);
				if(PelCol==0)
					PokePels+=TempPels;
				TempPels<<=1;
				}
			*pTempBmpBytes=PokePels;
			pTempBmpBytes++;
			}
		if(i/16!=((i/8)+1)/2)
			pTempBmpBytes++;
		if (statProc)
			{
			pct=(int)(j*50/BmpDims.y);
			if ((*statProc)(pct))
				{
				GlobalUnlock(hGlobal);
				GlobalFree(hGlobal);
				GlobalUnlock(hBmp);
				return(NULL);
				}
			}
		}

	for(j=0;j<BmpDims.y;j++)
		{
		for(i=0;i<BmpDims.x;i+=8)
			{
			TempPels=0x01;
			PokePels=0;
			for(k=0;k<8;k++)
				{
				PelCol=GetPixelCol((BITMAPINFOHEADER *)lpDib,lpDibBits,(int)(i+k),(int)j);
				if(PelCol==2)
					PokePels+=TempPels;
				TempPels<<=1;
				}
			*pTempBmpBytes=PokePels;
			pTempBmpBytes++;
			}
		if(i/16!=((i/8)+1)/2)
			pTempBmpBytes++;
		if (statProc)
			{
			pct=(int)(j*50/BmpDims.y+50);
			if ((*statProc)(pct))
				{
				GlobalUnlock(hGlobal);
				GlobalFree(hGlobal);
				GlobalUnlock(hBmp);
				return(NULL);
				}
			}
		}

	*nBmpBits=pTempBmpBytes-pBmpBytes;
	GlobalUnlock(hGlobal);
	GlobalUnlock(hBmp);
	return(hGlobal);
	}

BOOL SavePic(HBITMAP hBitmap,LPCSTR fileName,XMSTATPROC statProc,LPINT errorstate)
	{
	LONG		nBmpBits;
	HFILE		hFile;
	HGLOBAL		hGlobal;
	BYTE __huge	*pBmpBytes;

	hGlobal=ExtractBmpBits(hBitmap,&nBmpBits,statProc);
	InitPicFileHeader(nBmpBits,BmpDims);

	pBmpBytes=(unsigned char __huge *)GlobalLock(hGlobal);

	CalcCrcChksum(&ws_pic_header1.crc_bits_checksum,pBmpBytes,(int)nBmpBits/2);
	CalcCrcChksum(&ws_pic_header2.crc_bits_checksum,((LPBYTE)pBmpBytes)+nBmpBits/2,(int)nBmpBits/2);

	if((hFile=_lcreat(fileName,0))==HFILE_ERROR)                                 
		{
        *errorstate=6;
		return(FALSE);
		}

	if(HFILE_ERROR==_hwrite(hFile,(LPBYTE)&p_fsig,sizeof(p_fsig)))
		{
        *errorstate=8;
		_lclose(hFile);
		return(FALSE);
		}

	if(HFILE_ERROR==_hwrite(hFile,(LPBYTE)&ws_pic_header1,sizeof(ws_pic_header1)))
		{
        *errorstate=8;
		_lclose(hFile);
		return(FALSE);
		}

	if(HFILE_ERROR==_hwrite(hFile,(LPBYTE)&ws_pic_header2,sizeof(ws_pic_header2)))
		{
        *errorstate=8;
		_lclose(hFile);
		return(FALSE);
		}

	if (HFILE_ERROR==_hwrite(hFile,pBmpBytes,(LONG)nBmpBits))
		{
        *errorstate=8;
		_lclose(hFile);
		return(FALSE);
		}
	GlobalUnlock(hGlobal);
	GlobalFree(hGlobal);
		
	if(_lclose(hFile)==HFILE_ERROR)
		{
        *errorstate=9;
		return(FALSE);
		}
	else
		return(TRUE);
	}
