/*	CHILD.C
Child program to demo child printing to a parent's window
Written by NSM, June 1995
Copyright (C) Psion PLC 1995
*/
#include "intf.h"

GLREF_D TEXT *DatCommandPtr;

LOCAL_D WORD completeStat;
LOCAL_D COMMAND_LINE_INFO *pI;

LOCAL_C VOID panic(TEXT *aMess,INT anErr)
/*
Output error message and terminate program.
*/
    {
    TEXT buf[0x40];

    if (anErr)
		{
		p_errs(&buf[0],anErr);
		p_printf("Error: %s failed - %s",aMess,&buf[0]);
		}
    else
		p_printf("Error: %s",aMess);
    p_getch();
    p_exit(TRUE);
    }

LOCAL_C VOID writeToParent(VOID *aBuf,INT aLen)
/*
Write aBuf to the parent program.
*/
    {
	WORD zero;

	if (aLen>MAX_BUFFER)
		panic("Buffer size exceeded",0);
	completeStat=E_FILE_PENDING; /* Set the completion status to pending */
	p_pcpyto(pI->pid,pI->buffer,aBuf,aLen); /* Copy the data to the parent buffer */
	zero=0;
	p_pcpyto(pI->pid,pI->status,&zero,sizeof(zero)); /* Zero the parents status word */
	p_iosignalbypid(pI->pid); /* Signal the parent that a request is pending */
	p_waitstat(&completeStat); /* Wait till the parent has completed the request */
	}

LOCAL_C VOID parentPrintfSetup()
/*
Set up printing to the parent
*/
	{
	WORD *statusPtr;
	TEXT *pC;

	pC=DatCommandPtr+p_slen(DatCommandPtr)+1; /* Get to the command line */
	if (*pC!=sizeof(COMMAND_LINE_INFO)) /* Check we were run by the parent */
		panic("Not run by parent program",0);
	pI=(COMMAND_LINE_INFO *)(pC+1); /* Set global to point to the command line info */
	statusPtr=(&completeStat);
/*
The first thing we must write to the parent is the address of our
completion status word so that we can synchronize with our parent.
*/
	writeToParent(&statusPtr,sizeof(statusPtr));
	}

LOCAL_C VOID printf(TEXT *aFmt,...)
/*
Print a formatted string via the parent.
*/
    {
	INT len;
    TEXT buf[0x100];

    len=p_atob(&buf[0],aFmt,(&aFmt)+1);
    buf[len++]=0;
	writeToParent(&buf[0],len);
    }

GLDEF_C INT main()
/*
The main program.
*/
	{

	parentPrintfSetup(); /* Set up printing to the parent */
/*
Now test the printf function.
*/
	printf("Hello from the child");
	printf("My command line was:");
	printf("\"%s\"",&pI->restOfCommand[0]);
	printf("Last print from the child");
/*
Now we can just exit as normal.
*/
	return(0);
	}

