/*
  World time clock object
*/

#include <clock.h>

LOCAL_C INT clockType()
//
// Return the city clock type according to the OS version
//
{
 /* return(p_version()>=0x368F? WS_CLOCK_MEDIUM3:WS_CLOCK_MEDIUM2); ple */
 /* WS_CLOCK_MEDIUM3 not defined in v2.1 of C SDK                       */
  return(p_version()>=0x368F? WS_CLOCK_MEDIUM2:WS_CLOCK_MEDIUM2);
}

#pragma METHOD_CALL

METHOD VOID clock_type(PR_CLOCK *self,INT flags)
//
// Free the clock and create new one with given flags
//
// Used when changing from analog to digital
//
{
  if(self->clock.type!=flags)
  {
    WS_CREATE_CLOCK cl;

    wFree(self->clock.time_id);

    cl.id     = DatApp3->win.id;
    cl.type   = clockType();
    cl.pos.x  = self->clock.xpos;
    cl.pos.y  = self->clock.ypos;
    cl.offset = self->clock.offset;
    cl.flags  = flags;

    self->clock.time_id=wsCreateClock2(&cl,NULL);
    self->clock.type=flags;
  }
}

METHOD VOID clock_clock(PR_CLOCK *self,INT xpos,INT ypos,INT flags)
//
// Create two clocks: the time and the date below it
//
{
  WS_CREATE_CLOCK cl;

  cl.id     = DatApp3->win.id;
  cl.type   = clockType();
  cl.pos.x  = xpos;
  cl.pos.y  = ypos;
  cl.offset = 0;
  cl.flags  = flags;

  self->clock.type=flags;
  self->clock.xpos=xpos;
  self->clock.ypos=ypos;
  
  self->clock.time_id=wsCreateClock2(&cl,NULL);

  cl.type   = WS_CLOCK_FORMATTED;
  cl.pos.x  = xpos+28;
  cl.pos.y  = ypos+53;
  cl.offset = 0;
  cl.flags  = 0;
  cl.font   = FONT_ID_SWISS_8;
  cl.style  = G_STY_BOLD;
  cl.flags  = WS_CLOCK_CENTRED;

  self->clock.xpos=xpos;
  self->clock.ypos=ypos;
  
  self->clock.date_id=wsCreateClock2(&cl,"%*E %*D (%A)");
}

METHOD VOID clock_draw(PR_CLOCK *self)
//
// Redraw the clock name
//
{
  P_RECT area;
  TEXT *ptr;
  INT xpos,ypos;
  G_GC gc;

  xpos=self->clock.xpos;
  ypos=self->clock.ypos;

  ptr=self->clock.title;

  area.tl.x=xpos-13;
  area.br.x=xpos-13+85;
  area.br.y=ypos-1;
  area.tl.y=ypos-11;

  hSetGFont(FONT_ID_SWISS_8);
  hSetGStyle(G_STY_BOLD);

  gc.flags=0;

  gSetGC(WS_TEMPORARY_GC,G_GC_MASK_GREY,&gc);

  gPrintBoxText(&area,9,G_TEXT_ALIGN_CENTRE,0,ptr,p_slen(ptr));

  if(!self->clock.valid)
  {
    gc.flags=G_GC_FLAG_GREY_PLANE;

    gSetGC(WS_TEMPORARY_GC,G_GC_MASK_GREY,&gc);

    gClrRect(&area,G_TRMODE_CLR);
  }
  hSetGStyle(G_STY_NORMAL);
}

METHOD TEXT *clock_getcity(PR_CLOCK *self)
//
// Return the clocks name
//
{
  return(self->clock.cityname);
}

METHOD TEXT *clock_gettitle(PR_CLOCK *self)
//
// Return the clocks title
//
{
  return(self->clock.title);
}

METHOD INT clock_getoffset(PR_CLOCK *self)
//
// Return the clocks offset
//
{
  return(self->clock.offset);
}

METHOD VOID clock_city(PR_CLOCK *self,TEXT *city,TEXT *title)
//
// Set the clocks city and title
//
{
  p_scpy(self->clock.cityname,city);

  p_scpy(self->clock.title,title);
}

METHOD VOID clock_invalidate(PR_CLOCK *self)
//
// Invalidate the clock
//
{
  self->clock.valid=FALSE;
}

METHOD VOID clock_set(PR_CLOCK *self,INT offset)
//
// Set the clock and mark as valid
//
{
  wsSetClock(self->clock.time_id,offset);
  wsSetClock(self->clock.date_id,offset);

  self->clock.offset=offset;
  self->clock.valid=TRUE;
}

METHOD VOID clock_destroy(PR_CLOCK *self)
//
// Free the two clocks: time and date
//
{
  wFree(self->clock.time_id);
  wFree(self->clock.date_id);
}



