/*
  World alarm clock active objects
*/

#include <clock.h>

#pragma METHOD_CALL

METHOD VOID clockud_ao_queue(PR_CLOCKUD *self,INT clk)
//
// Queue the active object for updating a specified clock
//
{
  TEXT *buf;

  self->clockud.clk=clk;

  self->active.isactive=TRUE;

  buf=(TEXT *)p_send2(clocks[clk],O_GETCITY);

//
// Ask the world engine for data on the city for this clock
//
// This active object will complete when the world engine
// returns the information requested, and ao_run will be called
//

  p_ioc5(world,WR_FIND_CITY,&self->active.stat,buf,&self->clockud.find);
}

METHOD VOID clockud_ao_init(PR_CLOCKUD *self)
//
// Initialise the update clocks active object
//
{
  self->active.priority=PRIORITY_ACTIVE_COMPUTE;

  p_send3(w_am,O_AM_ADD_TASK,self);
}

METHOD VOID clockud_ao_cancel(PR_CLOCKUD *self)
//
// Cancel updating the clocks - cancel the request to the world engine
//
{
  p_iow2(world,P_FCANCEL);
  p_supersend2(self,O_AO_CANCEL);
}

METHOD INT clockud_ao_run(PR_CLOCKUD *self)
//
// Update the next clock
//
{
  WR_CITY_DATA result;
  INT clk,offset;
  E_CONFIG config;

  p_getctd(&config);

  clk=self->clockud.clk;

//
// Get the information for this clock from the world engine
//
  p_iow3(world,WR_GET_CITY_DATA,&result);
//
// Adjust for any gmt offsets, summertime etc
//
  offset=result.GMT-config.gmtOffset;

  if(E_DST_HOME&config.summerTime)
    offset-=60;

  if(result.DST&config.summerTime)
    offset+=60;
//
// Set the clock and redraw the clock name
//
  p_send3(clocks[clk],O_SET,offset);

  gCreateTempGC0(DatApp3->win.id);
  p_send2(clocks[clk],O_DRAW);
  gFreeTempGC();

//
// Queue the next clock to be updated
//

  if(++clk<NUM_CLOCKS)
	 p_send3(self,O_AO_QUEUE,clk);
  else
  {
    if(sort==SORT_BY_OFFSET)
      Sort(FALSE);
  }
  return(RUN_ACTIVE_USED);
}

METHOD VOID alarmao_ao_init(PR_ALARMAO *self)
//
// Initialise the alarm active object
//
{
  self->active.priority=PRIORITY_ACTIVE_COMPUTE;

  p_send3(w_am,O_AM_ADD_TASK,self);
}

METHOD VOID alarmao_ao_queue(PR_ALARMAO *self,ULONG *time)
//
// Queue the alarm at the given time
//
{
  AXDATA details;

  details.absTime=*time;
  details.dueTime=*time;

  hDlgSense(ALARM_SOUND,&details.sound);

  p_ioc5(alm,A_FTIMED_X,&self->active.stat,&details,&alarmText);

  self->active.isactive=TRUE;
}

METHOD INT alarmao_ao_run(PR_ALARMAO *self)
//
// The alarm has gone off
//
{
  if (self->active.stat==E_FILE_CANCEL)
    self->active.stat=0;

  return(RUN_ACTIVE_USED);
}

METHOD VOID alarmao_ao_cancel(PR_ALARMAO *self)
//
// Cancel any pending alarm
//
{
  p_iow2(alm,P_FCANCEL);

  p_supersend2(self,O_AO_CANCEL);
}



