/*
  World time dialogs
*/

#include <clock.h>
#include <chlist.g>

LOCAL_C VOID SetClock(INT clk,WR_CITY_DATA *result,TEXT *title)
//
// Set the given clock using the city and title supplied
//
{
  p_send4(clocks[clk],O_CITY,result->f.city,title);

  Sort(TRUE);
}

LOCAL_C PR_VAFLAT* CreateList(VOID)
//
// Create a list of clock titles
//
{
  INT clk;
  PR_VAFLAT *list;

  p_send4(list=(PR_VAFLAT *)f_new(CAT_CLOCK_OLIB,C_VAFLAT),O_VA_INIT,28,8);

  for(clk=0;clk<NUM_CLOCKS;clk++)
    p_send3(list,O_VA_APPEND,(TEXT*)p_send2(clocks[clk],O_GETTITLE));

  return(list);
}

LOCAL_C VOID GetCityTime(ULONG time,INT clk,INT inc,P_DAYSEC *daysec)
{
  LONG ofst;

  if(clk<0)
    ofst=-(p_send3(clocks[clk+NUM_CLOCKS],O_GETOFFSET,clk+NUM_CLOCKS)*60l);
  else
    ofst=p_send3(clocks[clk],O_GETOFFSET,clk)*60l;

  time+=ofst;

  if(inc)
  {
    time-=time%60;
	 time+=60;
  }
  p_sttods(&time,daysec);
}

#pragma METHOD_CALL

METHOD VOID alarmdl_dl_changed(PR_ALARMDL *self,INT index)
{
  INT clk;
  P_DAYSEC daysec;

  clk=hDlgSenseChlist(ALARM_CITY);
  
  GetCityTime(p_date(),clk,TRUE,&daysec);

  hDlgSetDtedit(ALARM_TIME,daysec.sec);
  hDlgSetDtedit(ALARM_DATE,daysec.day);
}

METHOD VOID alarmdl_dl_dyn_init(PR_ALARMDL *self)
{
  SE_CHLIST chlist;

  self->alarmdl.list=CreateList();

  chlist.data=(PR_VAROOT *)self->alarmdl.list;
  chlist.nsel=0;
  chlist.set_flags=SE_CHLIST_DATA|SE_CHLIST_NSEL;

  hDlgSet(ALARM_CITY,&chlist);

  p_send2(self,O_DL_CHANGED);
}

METHOD INT alarmdl_dl_key(PR_ALARMDL *self,INT index,INT key)
{
  INT clk;
  ULONG time;
  LONG ofst;
  P_DAYSEC daysec;
 
  switch(key)
  {
    case W_KEY_MENU:
		hDlgSet(ALARM_SOUND,NULL);

		return(WN_KEY_NO_CHANGE);
	 case W_KEY_ESCAPE:
		return(WN_KEY_CHANGED);
  }
  clk=hDlgSenseChlist(ALARM_CITY);

  p_scpy(alarmText,hDlgSenseEdwin(ALARM_TEXT));

  // get time to set alarm at

  daysec.day=hDlgSenseDtedit(ALARM_DATE);
  daysec.sec=hDlgSenseDtedit(ALARM_TIME);

  p_dstost(&daysec,&time);

  // subtract offset

  ofst=p_send3(clocks[clk],O_GETOFFSET,clk)*60l;

  time=time-ofst;

  if(time<=p_date())
  {
	 wInfoMsg("Cannot set alarm in past");

	 return(WN_KEY_NO_CHANGE);
  }
  else
  {
    p_send3(alarm,O_AO_QUEUE,&time);

    wInfoMsg("Alarm set");

    wFlush();
  }
  return(WN_KEY_CHANGED);
}

METHOD VOID setupdl_dl_dyn_init(PR_SETUPDL *self)
{
  SE_CHLIST chlist;

  self->setupdl.list=CreateList();

  chlist.data=(PR_VAROOT *)self->setupdl.list;
  chlist.nsel=0;
  chlist.set_flags=SE_CHLIST_DATA|SE_CHLIST_NSEL;

  hDlgSet(1,&chlist);

  p_send3(self,O_DL_CHANGED,1);
}

METHOD INT setupdl_dl_key(PR_SETUPDL *self,INT index,INT key)
{
  SENSE_WLDSELWN sense;
  INT clk,num;

  clk=hDlgSenseChlist(1);

  hDlgSense(2,&sense);

  for(num=0;num<NUM_CLOCKS;num++)
  {
    if(p_scmp(sense.data.ci.f.city,(TEXT *)p_send2(clocks[num],O_GETCITY))==0)
	 {
	   if(num!=clk)
		{
	     wInfoMsg("Clock already exists");

		  return(WN_KEY_NO_CHANGE);
		}
	 }
  }
  SetClock(clk,&sense.data.ci,(TEXT*)hDlgSenseEdwin(3));

  return(WN_KEY_CHANGED);
}

METHOD VOID setupdl_dl_changed(PR_SETUPDL *self,INT index)
{
  INT clk;
  TEXT *buf;
  WR_FIND_RES find;

  clk=hDlgSenseChlist(index);

  if(index==1)
  {
    buf=(TEXT*)p_send2(clocks[clk],O_GETCITY);

    p_iow4(world,WR_FIND_CITY,buf,&find);

    hDlgSet(2,&find);

    hDlgSetEdwin(3,(TEXT*)p_send2(clocks[clk],O_GETTITLE));
  }
  else
  {
    SENSE_WLDSELWN sense;
 
    hDlgSense(2,&sense);

    hDlgSetEdwin(3,sense.data.ci.f.city);
  }
}

METHOD INT calcdl_dl_key(PR_CALCDL *self)
{
  return(WN_KEY_CHANGED);
}

METHOD INT calcdl_wn_key(PR_CALCDL *self,INT key,INT mod)
{
  INT ret;
  INT index;

  ret=p_supersend4(self,O_WN_KEY,key,mod);
 
  index=self->dlgbox.current;

  if(index==HOMETIME || index==CITYTIME)
    p_send3(self,O_DL_CHANGED,index);

  return(ret);
}

METHOD INT calcdl_dl_take_focus(PR_CALCDL *self,INT index)
{
  INT ret,oldindex;

  oldindex=self->dlgbox.current;

  ret=p_supersend3(self,O_DL_TAKE_FOCUS,index);

  if(ret)
	 p_send3(self,O_DL_CHANGED,oldindex);

  return(ret);
}

METHOD VOID calcdl_dl_dyn_init(PR_CALCDL *self)
{
  SE_CHLIST chlist;

  self->calcdl.list=CreateList();

  chlist.data=(PR_VAROOT *)self->calcdl.list;
  chlist.nsel=0;
  chlist.set_flags=SE_CHLIST_DATA|SE_CHLIST_NSEL;

  hDlgSet(CALC1,&chlist);

  hDlgSetDtedit(HOMETIME,p_date());

  p_send2(self,O_DL_CHANGED);
}

METHOD VOID calcdl_dl_changed(PR_CALCDL *self,INT index)
{
  ULONG time;
  INT clk;
  P_DAYSEC daysec;

  clk=hDlgSenseChlist(CALC1);

  // allow for any negative offsets
  // bearing in mind we are dealing with times only
  // and not days

  time=86400l;

  switch(index)
  {
    case HOMETIME:
	 case CALC1:
      time+=hDlgSenseDtedit(HOMETIME);

		GetCityTime(time,clk,FALSE,&daysec);

		hDlgSetDtedit(CITYTIME,daysec.sec);
		break;
	 case CITYTIME:
		time+=hDlgSenseDtedit(CITYTIME);

		GetCityTime(time,(clk-NUM_CLOCKS),FALSE,&daysec);

		hDlgSetDtedit(HOMETIME,daysec.sec);
		break;
  }
}

METHOD VOID prefsdl_dl_dyn_init(PR_PREFSDL *self)
{
  hDlgSetChlist(2,sort);

  if(type==WS_CLOCK_FORCE_ANALOG)
    hDlgSetChlist(1,0);
  else
    hDlgSetChlist(1,1);
}

METHOD INT prefsdl_dl_key(PR_PREFSDL *self,INT key)
{
  sort=hDlgSenseChlist(2);

  if(hDlgSenseChlist(1)==0)
    type=WS_CLOCK_FORCE_ANALOG;
  else
    type=WS_CLOCK_FORCE_DIGITAL;

  return(WN_KEY_CHANGED);
}


