/*
  World alarm clock main
*/

#include <clock.h>

GLDEF_D PR_CLOCK *clocks[NUM_CLOCKS];
GLDEF_D PR_CLOCKUD *update;
GLDEF_D PR_ALARMAO *alarm;

GLDEF_D VOID *world;
GLDEF_D VOID *alm;

GLDEF_D TEXT filename[P_FNAMESIZE];

GLDEF_D INT homeTime;
GLDEF_D INT homeDate;
GLDEF_D INT sort;
GLDEF_D INT type;

GLDEF_D TEXT homeCity[22];

GLDEF_D TEXT alarmText[64];

GLDEF_C VOID main(VOID)
  {
    IN_HWIMMAN app;
    IN_WSERV wserv;

#ifndef EPOC
    GLREF_D P_DEVICE p_file;
    p_inst(&p_file,NULL);
#endif
    p_linklib(0);
    app.flags = FLG_APPMAN_RSCFILE
		    | FLG_APPMAN_CLEAN
		    | FLG_APPMAN_SRSCFILE
		    | FLG_APPMAN_FULLSCREEN;
    app.wserv_cat = wserv.com_cat = p_getlibh(CAT_CLOCK_CLOCK);
    app.wserv_class = C_CLOCKWS;
    wserv.com_class = C_CLOCKCM;

    p_send4(p_new(CAT_CLOCK_HWIM,C_HWIMMAN),O_AM_INIT,&app,&wserv);
    }

GLDEF_C VOID Sort(INT flag)
//
// Sort the clocks according to the type in 'sort'
//
// If flag is TRUE, queue the clocks for updating
//
{
  INT c,s;

  TEXT cities[NUM_CLOCKS][22];
  TEXT titles[NUM_CLOCKS][22];
  
  INT offset[NUM_CLOCKS];
//
// Get clock names and offsets...
//
  for(c=0;c<NUM_CLOCKS;c++)
  {
    p_scpy(cities[c],(TEXT *)p_send2(clocks[c],O_GETCITY));
    p_scpy(titles[c],(TEXT *)p_send2(clocks[c],O_GETTITLE));

    offset[c]=p_send2(clocks[c],O_GETOFFSET);
  }
//
// Sort them...
//
  do
  {
    s=FALSE;

    for(c=0;c<NUM_CLOCKS-1;c++)
    {
      INT swap;

      switch(sort)
      {
	case SORT_BY_NAME:
	  swap=(p_scmp(titles[c],titles[c+1])>0);
			 break;
		  case SORT_BY_OFFSET:
			 swap=(offset[c]>offset[c+1] && !flag);
			 break;
		  case SORT_BY_NEG_OFFSET:
			 swap=(offset[c]<offset[c+1] && !flag);
			 break;
		}
		if(swap)
		{
	INT o;

	s=TRUE;

	p_bswap(titles[c],titles[c+1],22);
	p_bswap(cities[c],cities[c+1],22);

	o=offset[c];
	offset[c]=offset[c+1];
	offset[c+1]=o;
      }
    }
  } while(s);

  for(c=0;c<NUM_CLOCKS;c++)
  {
    p_send4(clocks[c],O_CITY,cities[c],titles[c]);

    p_send2(clocks[c],O_INVALIDATE);
  }
  if(flag)
  {
    p_send2(update,O_AO_CANCEL);
    p_send3(update,O_AO_QUEUE,0);
  }
  else
  {
    gCreateTempGC0(DatApp3->win.id);

    for(c=0;c<NUM_CLOCKS;c++)
    {
      p_send3(clocks[c],O_SET,offset[c]);

      p_send2(clocks[c],O_DRAW);
    }
    gFreeTempGC();
  }
}

LOCAL_C INT LoadConfig()
{
  INT ret,d,clk;
  VOID *fcb;
  TEXT drv[4];

  hLoadResBuf(DEFAULT_CONFIG,filename);

  p_scpy(drv,"BAM");

  for(d=0;d<3;d++)
  {
    filename[5]=drv[d];    

    ret=p_open(&fcb,filename,P_FSTREAM|P_FRANDOM|P_FOPEN);

    if(ret==0)
    {
      for(clk=0;clk<NUM_CLOCKS;clk++)
      {
		  TEXT name[22],city[22];

	     p_read(fcb,city,22);
	p_read(fcb,name,22);

	p_send4(clocks[clk],O_CITY,city,name);
      }
      ret=p_read(fcb,(TEXT*)&sort,2);
      ret=p_read(fcb,(TEXT*)&type,2);

      if(sort>SORT_BY_NEG_OFFSET || sort<0)
	sort=SORT_BY_NAME;

      if(type!=WS_CLOCK_FORCE_ANALOG && type!=WS_CLOCK_FORCE_DIGITAL)
	type=WS_CLOCK_FORCE_ANALOG;

	   p_close(fcb);

		return(FALSE);
	 }
  }
  return(TRUE);
}

LOCAL_C INT clockTypeLarge()
//
// Return the large clock type according to the OS version
//
{
  /* return(p_version()>=0x368F? WS_CLOCK_XL2_ANALOG:WS_CLOCK_XL_ANALOG); */
  /* WS_CLOCK_XL2_ANALOG not found in v2.1 C SDK */
  return(p_version()>=0x368F? WS_CLOCK_XL_ANALOG:WS_CLOCK_XL_ANALOG);
}

LOCAL_C VOID CreateMainClock(VOID)
//
// Create the main home town clock
//
{
  WS_CREATE_CLOCK cl;

  cl.id     = DatApp3->win.id;
  cl.type   = clockTypeLarge();

  /* WS_CLOCK_XL2_ANALOG not found in v2.1 C SDK */
  /* if(clockTypeLarge()==WS_CLOCK_XL2_ANALOG)
  {
    cl.pos.x  = 14;
    cl.pos.y  = 25;
  }
  else
  {  */ 
    cl.pos.x  = 18;
    cl.pos.y  = 28;
  /* } */
  cl.offset = 0;
  cl.flags  = WS_CLOCK_WITH_SECONDS;
  
  homeTime=wsCreateClock2(&cl,NULL);

  cl.id     = DatApp3->win.id;
  cl.type   = WS_CLOCK_FORMATTED;
  
  /* WS_CLOCK_XL2_ANALOG not found in v2.1 C SDK */
  /* if(clockTypeLarge()==WS_CLOCK_XL2_ANALOG)
  {
    cl.pos.x  = 68;
    cl.pos.y  = 138;
  }
  else
  {   */
    cl.pos.x  = 68;
    cl.pos.y  = 133;
  /* } */
  cl.offset = 0;
  cl.font   = FONT_ID_SWISS_13;
  cl.style  = G_STY_BOLD;
  cl.flags  = WS_CLOCK_CENTRED;
  
  homeDate=wsCreateClock2(&cl,"%*E %*D (%A)");
}

LOCAL_C VOID SetMainClockName(VOID)
//
// Get home city name
//
{
  WR_FIND_RES find;

  p_iow3(world,WR_GET_HOME,&find);

  p_scpy(homeCity,find.city); 
}

#pragma METHOD_CALL

METHOD VOID clockws_ws_foreground(PR_CLOCKWS *self)
//
// Re-start the active object to update all clocks
// incase summer time or GMT offsets have changed  
// while we were in the background
//
{  
  SetMainClockName();

  p_send2(update,O_AO_CANCEL);

  p_send3(update,O_AO_QUEUE,0);
}

METHOD VOID clockws_ws_background(PR_CLOCKWS *self)
//
// Cancel any updating of the clocks and invalidate them
//
{
  INT clk;

  p_send2(update,O_AO_CANCEL);

  for(clk=0;clk<NUM_CLOCKS;clk++)
    p_send2(clocks[clk],O_INVALIDATE);
}

METHOD VOID clockws_ws_dyn_init(PR_CLOCKWS *self)
{
  p_send2(DatApp3=(PR_CLOCKWN *)f_new(CAT_CLOCK_CLOCK,C_CLOCKWN),O_WN_INIT);

  p_send2(update=(PR_CLOCKUD *)f_new(CAT_CLOCK_CLOCK,C_CLOCKUD),O_AO_INIT);

  p_send2(alarm=(PR_ALARMAO *)f_new(CAT_CLOCK_CLOCK,C_ALARMAO),O_AO_INIT);

  self->wserv.cli =(PR_WIN *)DatApp3;

  self->wserv.help_index_id = H_HELP;
}

METHOD VOID clockwn_wn_init(PR_CLOCKWN *self)
{
  W_WINDATA wd;
  INT clk,xpos,ypos;

  self->win.flags=0;

  wd.background = W_WIN_BACK_CLR|W_WIN_BACK_GREY_CLR;

  wd.flags = 0;

  wd.extent.tl.x = wd.extent.tl.y = 0;

  wd.extent.width  = 480;
  wd.extent.height = 160;

  p_send5(self,O_WN_CONNECT,NULL_D,W_WIN_ALL_FLAGS|W_WIN_EXTENT|W_WIN_BACKGROUND,&wd);

  hInitVis(self);
//
// Open the world engine and alarm server
//
  p_open(&world,"WLD:",0xffff);

  p_open(&alm,"ALM:",0xffff);

//
// Create the main home clock and set it
//
  CreateMainClock();

  SetMainClockName();

  wsSetList(W_STATUS_WINDOW_ICON,NULL,0);

  xpos=145;
  ypos=16;

  type=WS_CLOCK_FORCE_ANALOG;
  sort=SORT_BY_NAME;

//
// Create the clocks using the default cities and names
//
  for(clk=0;clk<NUM_CLOCKS;clk++)
  {
    clocks[clk]=f_new(CAT_CLOCK_CLOCK,C_CLOCK);

	 if(clk==4)
	 {
		xpos=145;
		ypos+=76;
	 }
	 p_send5(clocks[clk],O_CLOCK,xpos,ypos,type);

	 xpos+=86;
  }
  if(LoadConfig())
  {
    for(clk=0;clk<NUM_CLOCKS;clk++)
    {
	   TEXT name[22],city[22];
 
      hLoadResBuf(TITLES0+clk,name);
      hLoadResBuf(CITIES0+clk,city);

      p_send4(clocks[clk],O_CITY,city,name);
    }
  }
  for(clk=0;clk<NUM_CLOCKS;clk++)
    p_send3(clocks[clk],O_TYPE,type);
}

METHOD VOID clockwn_wn_draw(PR_CLOCKWN *self)
//
// Redraw all the clock names and the lines between the clocks
//
{
  P_RECT area;
  INT clk,xpos;
  G_GC gc;

  area.tl.x=18;
  area.br.x=118;
  
  /* WS_CLOCK_XL2_ANALOG not found in v2.1 C SDK */
  /* if(clockTypeLarge()==WS_CLOCK_XL2_ANALOG)
  {
    area.tl.y=9;
    area.br.y=26;
  }
  else
  {  */
    area.tl.y=10;
    area.br.y=27;
  /* } */
  hSetGFont(FONT_ID_SWISS_13);
  hSetGStyle(G_STY_BOLD);

  gPrintBoxText(&area,12,G_TEXT_ALIGN_CENTRE,0,homeCity,p_slen(homeCity));
  
  gBorder2(W_BORDER_TYPE_1,W_BORD_CORNER_2|W_BORD_SHADOW_ON);
 
  gc.flags=G_GC_FLAG_GREY_PLANE;

  gSetGC(WS_TEMPORARY_GC,G_GC_MASK_GREY,&gc);

  gDrawLine(131,80,472,80);

  for(xpos=131,clk=0;clk<NUM_CLOCKS;clk++,xpos+=86)
    gDrawLine(xpos,8,xpos,152);

  for(clk=0;clk<NUM_CLOCKS;clk++)
    p_send2(clocks[clk],O_DRAW);

  wFlush();
}

METHOD VOID clockwn_wn_emphasise(PR_CLOCKWN *self,INT flag)
{
// Do nothing. The default wn_emphasise invalidates the whole window
// and will cause a flicker when menu or dialogs are used

  self->win.flags&=~(PR_WIN_EMPHASISED);
  if(flag)
    self->win.flags|=(PR_WIN_EMPHASISED);
}

