/* Database Example program.*/
/* Heather Chia */
/* 02-09-91 */

#include <plib.h>
#include <p_dbf.h>


#define COMPRESS

#define MAIN_BUFFER_LENGTH 4096           /* 4K Must be >=512 and <=16384 */

GLDEF_C VOID aFindRoutine(TEXT *);        /* Subroutine which uses DbfFindRead() */
LOCAL_D UBYTE Buffer[MAIN_BUFFER_LENGTH]; /* Buffer into which Dbf funcs will use */
LOCAL_D VOID *channel;                    /* currently open dbf channel */

/* */

/* */

GLDEF_C INT main()
{
   DbfHeader pHeader;
   INT State;
   INT ret;
   UBYTE *pB;
   TEXT str[E_MAX_ERROR_TEXT_SIZE];
   UWORD offset;
   UINT recno;

   
   /* USE this if you want to be compatible with PSION products generally */
   /* In particular OPL */
   p_scpy((TEXT *)pHeader.fileType,"OPLDatabaseFile"); /* work's cos it's exactly 15 bytes */

   pHeader.createVersion=DbfVersion();
   pHeader.dataStart=DbfHeaderLength; /* NO extended header info */
   pHeader.needVersion=DbfVersion();

   pHeader.firHeader=((DbfRecordTypeFir<<12)|3); /* Records will contain three fields */
   pHeader.fir[0]=DbfFieldTypeString; /* TWO STRINGS             */
   pHeader.fir[1]=DbfFieldTypeString;
   pHeader.fir[2]=DbfFieldTypeWord;   /* WORD field              */

   channel = NULL_D;
   State=DbfStateStart;
/* This is the correct way to do this if you don't want to be hung in the open */
/* for a long time. Each time this is called it reads one buffer's worth from the file and */
/* scans  that buffer for the start of the records so that the index can be built. Ths index */
/* is used to speed up e.g DbfAbsRead be making sure it doen't have to scan */
/* from the start of the file every time */
/* Each time round this loop the buffer length is read from the file and */
/* the index build for that section of the file */
    
   do
   {
      /* The 1 at the end is the type of standard data records */
      ret=DbfOpen(&State,&channel,"text.txt",P_FCREATE|P_FUPDATE,&pHeader,&Buffer[0],MAIN_BUFFER_LENGTH,1);
      p_printf("Opening Database...");     /* Something like Opening file .... is nice here printing an extra . each time round */
   } while (!ret && State!=DbfStateStart); /* Stops either on an error OR when State becomes = DbfStateStart again i.e. finished */

   if (ret >= 0)
   {
      p_notify("Returns 0","Success",NULL,NULL,NULL);
   }
   else
   {
      /* p_errs(&str[0],ret);*/
      str[p_itob(str,ret)]='\0';      
      p_notify(str,"","BLAST","BANG","FRIG");
      p_exit(ret);
   }
   p_notify("Open"," returned ",NULL,NULL,NULL);


/*
Now want to append a record with the two string fields set to "Heather's"
and "First" and word field of 1.
Two bytes must always be left free at the start of the buffer for the Dbf code
to put the type and length of the record vefore writing to the file
so...
*/
    pB=(&Buffer[2]);
    *pB++=9;                     /* length of Heather's */
    pB=p_bcpy(pB,"Heather's",9);
    *pB++=5;                     /* length of first */
    pB=p_bcpy(pB,"First",5);
    *(UWORD *)pB=1;              /* Word long and double fields just contain
                                    the value */
    pB+=sizeof(UWORD);

/* There is no need to DbfTrash before and append - the code is smart enough */
/* to realise that you must have put something in the buffer to want to append it */
   if ((ret=DbfAppend(channel,pB-&Buffer[2]))!=0)
   {
      p_notifyerr(ret,"DbfAppend Error",NULL,NULL,NULL); /* Do some wonderfully friendly error handling here */
   }

   pB=(&Buffer[2]);              /* Get back to beginning of Buffer */
   *pB++=14;
   pB=p_bcpy(pB,"Another Record",14);
   *pB++=5;
   pB=p_bcpy(pB,"Zippy",5);
   *(UWORD *)pB=10;
   pB+=sizeof(UWORD);

   if ((ret=DbfAppend(channel,pB-&Buffer[2]))!=0)
   {
      p_notifyerr(ret,"DbfAppend Error",NULL,NULL,NULL); /* Do some wonderfully friendly error handling here */
   }
   else
      p_notify("Appending 2 records","was successful",NULL,NULL,NULL);


   DbfClose(channel);
   State = DbfStateStart;
   do
   {
      ret=DbfOpen(&State,&channel,"text.txt",P_FOPEN|P_FUPDATE,&pHeader,&Buffer[0],
      MAIN_BUFFER_LENGTH,1);
      if (ret<0)
      {
         p_notifyerr(ret, "DbfOpen failed", "SNAP","CRACKLE","POP");
         p_exit(ret);
      }
   } while (State!=DbfStateStart);
   
   pB=(&Buffer[2]);              /* Get back to beginning of Buffer */
   *pB++=14;
   pB=p_bcpy(pB,"Crazy horse   ",14);
   *pB++=5;
   pB=p_bcpy(pB,"hippo",5);
   *(UWORD *)pB=22;
   pB+=sizeof(UWORD);

   State = DbfStateStart;
   do
   {
      ret=DbfUpdate(&State,channel,pB-&Buffer[2]);
      if (ret<0)
      {
         p_notifyerr(ret, "DbfUpdate failed", "SNAP","CRACKLE","POP");
         p_exit(ret);
      }
   } while (State!=DbfStateStart);

   ret = DbfCount(channel);
   p_printf("Number of recs %d",ret);
   p_getch();

 
   ret = DbfFirstRead(channel,&offset);
   if (ret<0)
   {
      p_notifyerr(ret, "DbfFirstRead failed", "SNAP","CRACKLE","POP");
      p_exit(ret);
   }
   else
      p_notify("First Read Record length","","EENY","MEENY","MINEY");

   State = DbfStateStart;
   do
   {
      ret = DbfEraseRead(&State,channel,&offset);
      if (ret <0)
      {
         p_notifyerr(ret, "DbfEraseRead failed", "SNAP","CRACKLE","POP");
         p_exit(ret);
      }
      else
         p_notify("Erasing the first","record was fine","SNAP","CRACKLE","POP");

   } while (State!=DbfStateStart);

   aFindRoutine("Heather");
   
   aFindRoutine("hippo");
   
#ifdef COMPRESS

   recno = DbfSense(channel);
   str[p_itob(str,(INT)recno)] = '\0';

   p_notify("DbfSense Current record",str,0,0,0);

   State = DbfStateStart;
   do
   {
      ret=DbfCompress((UINT *)&State,channel);
      if (ret<0)
      {
         p_notifyerr(ret, "DbfCompress failed", "SNAP","CRACKLE","POP");
         p_exit(ret);
      }
      else
         p_notify("Compression of Database","OK","SNAP","CRACKLE","POP");
   }while (State != DbfStateStart);

   recno = DbfSense(channel);
   str[p_itob(str,(INT)recno)] = '\0';

   p_notify("Now DbfSense Current record",str,0,0,0);

#endif

   return(0);

}


GLDEF_C VOID aFindRoutine(TEXT *pSeaStr)
/*
Searches forwards from the current position for the passed search string
Find may also take an inordinately long time.
*/
{
   UBYTE *pB,*pBs;
   UINT state;
   INT    ret;
   UWORD offset;

   state=DbfStateStart;


   p_notify("Seastr is",pSeaStr,NULL,NULL,NULL);

   do
   {
      ret=DbfFindRead(&state,
                     channel,
                     pSeaStr,
                     p_slen(pSeaStr),
                     DbfFindCaseIndependent|DbfFindFirst|255,
                     DbfFindAllStrings,
                     &offset);  
      } while (ret>=0 && state!=DbfStateStart); /* stops on an error or when done */


   if (ret<0)
   {
      if (ret==E_FILE_EOF) /* just means you failed to find */
                           /* what you were looking for     */
                               /* nothing fatal */
      {
         p_notifyerr(ret,"String not found","Whizzy","Willy","Wow"); /* tell them */
      }
      else
      {
         p_notifyerr(ret,"DbfFindRead Error","Bizzy","Biddy","Bow");  /* somethings really gone wrong */
         p_exit(ret);
      }
   }
   else                             /* Found a match ok */
   {
      /** NB the offset marks the beginning of the data NOT including the header */
        pB=&Buffer[offset];           /* the start of the data in the buffer */

        DbfCopyDown(channel,(UWORD)offset);
        p_printf("%s",Buffer);
        p_getch();
   }
}
