//
// File pictobmp.cpp
// Functions for converting a Psion picture to a Windows device-independent bitmap
//
// Written by LCR, January 1995
// (C) Copyright Psion PLC 1995
//

#define WINDOWS_LEAN_AND_MEAN
#include <windows.h>
#include "psiwinxl.h"

typedef struct
	{
	char app_id[3];
	BYTE app_id_chksum;
	BYTE file_ver;
	BYTE app_ver;
	WORD number_of_bitmaps;
	} P_FSIG;

typedef struct
	{
	WORD crc_bits_checksum;
	WORD x_pels;
	WORD y_pels;
	WORD bitmap_bytes;
	LONG offset;
	} WS_PIC_HEADER;

HGLOBAL LoadPic (LPCSTR szFileName,LPINT errorstate)
    {
	WORD				i,maxbmps;
	LPWORD				nbmps;
	BYTE __huge			*pPicHead;
	BYTE __huge			*pPic;
	BYTE __huge			*pTemp;
    HFILE				hFile ;
	P_FSIG				p_fsig;
	HGLOBAL				hPicHead,hPic,hTemp;
	WS_PIC_HEADER FAR	*WsPicHead;

    if ((hFile=_lopen(szFileName,OF_READ))==HFILE_ERROR)
    	  {
    	  *errorstate=1;
          return(NULL);
          }

	if(HFILE_ERROR==_hread(hFile,&p_fsig,sizeof(P_FSIG)))
        {                                                   
        *errorstate=2;
        _lclose(hFile);
        return(NULL);
        }

	maxbmps=p_fsig.number_of_bitmaps;
	if(maxbmps>2)
		maxbmps=2; /* restrict number of bitmaps loaded to first two-black plane & grey plane */
    hPicHead=GlobalAlloc(GMEM_MOVEABLE,((DWORD)sizeof(WORD)+(DWORD)maxbmps*(DWORD)sizeof(WS_PIC_HEADER)));
    if (hPicHead==NULL)
         {
         *errorstate=3;
         _lclose(hFile) ;
         return(NULL);
         }
	pPicHead=(unsigned char __huge *)GlobalLock(hPicHead);
	WsPicHead=(WS_PIC_HEADER FAR *)(sizeof(WORD)+pPicHead);
	nbmps=(LPWORD)pPicHead;
	*nbmps=maxbmps;

   if(HFILE_ERROR==_hread(hFile,WsPicHead,(long)(*nbmps)*(long)sizeof(WS_PIC_HEADER)))
        {
        *errorstate=2;
        _lclose(hFile) ;
 		GlobalUnlock(hPicHead);
		GlobalFree(hPicHead);
        return(NULL);
        }

   if(WsPicHead->bitmap_bytes>16383)
        {
        *errorstate=4;
        _lclose(hFile) ;
 		GlobalUnlock(hPicHead);
		GlobalFree(hPicHead);
        return(NULL);
        }

	if(*nbmps==2)
		if(WsPicHead->bitmap_bytes!=(WsPicHead+1)->bitmap_bytes)
			*nbmps=1; /* if second bitmap and it doesn't appear to be grey plane, lose it */

	if(p_fsig.number_of_bitmaps>maxbmps)
		{
		hTemp=GlobalAlloc(GMEM_MOVEABLE,(DWORD)(p_fsig.number_of_bitmaps-maxbmps)*(DWORD)sizeof(WS_PIC_HEADER));
		pTemp=(unsigned char __huge *)GlobalLock(hTemp);
		if(HFILE_ERROR==_hread(hFile,pTemp,(long)(p_fsig.number_of_bitmaps-maxbmps)*(long)sizeof(WS_PIC_HEADER)))
	        {
            *errorstate=2;
	        _lclose(hFile) ;
	 		GlobalUnlock(hPicHead);
			GlobalFree(hPicHead);
			GlobalUnlock(hTemp);
			GlobalFree(hTemp);
	        return(NULL);
	        }
		GlobalUnlock(hTemp);
		GlobalFree(hTemp);
		}

    hPic=GlobalAlloc(GMEM_MOVEABLE, ((DWORD)sizeof(WORD) + (DWORD)(*nbmps)*((DWORD)sizeof(WS_PIC_HEADER)+(DWORD)(WsPicHead->bitmap_bytes)) ));
    if(hPic==NULL)
        {
        *errorstate=3;
        _lclose(hFile) ;
        return(NULL);
        }
	pPic=(unsigned char __huge *)GlobalLock(hPic);
	
	for(i=0;i<(WORD)(sizeof(WORD)+((long)(*nbmps)*sizeof(WS_PIC_HEADER)));i++)
		*(pPic+i)=*((LPBYTE)pPicHead+i);

	GlobalUnlock(hPicHead);
	GlobalFree(hPicHead);
	nbmps=(LPWORD)pPic;
	WsPicHead=(WS_PIC_HEADER FAR *)(sizeof(WORD)+pPic);

	pPic+=sizeof(WORD)+(DWORD)(*nbmps)*sizeof(WS_PIC_HEADER);
	if (HFILE_ERROR==_hread(hFile,pPic, ((long)(*nbmps))*(long)(WsPicHead->bitmap_bytes) ))
		{
        *errorstate=2;
		_lclose(hFile);
		GlobalUnlock(hPic);
		GlobalFree(hPic);
		return(NULL);
		}
    _lclose(hFile);
	GlobalUnlock(hPic);
    return(hPic);
    }
	
HGLOBAL ConvertPicToDib(HGLOBAL hPic,XMSTATPROC statProc,LPINT errorstate)
	{                       
	int						pct;                                    
	long					bTotal;
	BOOL					low_nibble;
	BYTE					mask,set_white,count;
	WORD					i,j,bmpX_pels,bmpY_pels;
	DWORD					MemSize,bOffSet;
	BYTE __huge				*pPic;
	BYTE __huge				*pMem;
	BYTE __huge				*pDibBits;
	LPWORD					nbmps;
	HGLOBAL					hMem;
	LPRGBQUAD				Quad1,Quad2,Quad3;
	WS_PIC_HEADER FAR		*pWsHeader1;
	WS_PIC_HEADER FAR		*pWsHeader2;
	LPBITMAPINFOHEADER		pBmpHeader;

	pPic=(unsigned char __huge *)GlobalLock(hPic);
	nbmps=(LPWORD)pPic;
	pWsHeader1=(WS_PIC_HEADER FAR *)(pPic+sizeof(WORD));
	pWsHeader2=pWsHeader1+(*nbmps-1);
	pPic=(LPBYTE)pWsHeader1+(*nbmps)*sizeof(WS_PIC_HEADER);
	bmpX_pels=pWsHeader1->x_pels;
	bmpY_pels=pWsHeader1->y_pels;

	MemSize=((DWORD)pWsHeader1->bitmap_bytes)*4+(DWORD)(sizeof(RGBQUAD)*3)+(DWORD)sizeof(BITMAPINFOHEADER);
	hMem=GlobalAlloc(GHND,MemSize);
	pMem=(unsigned char __huge *)GlobalLock(hMem);
	pBmpHeader=(LPBITMAPINFOHEADER)pMem;
	Quad1=(LPRGBQUAD)((LPBYTE)pMem+sizeof(BITMAPINFOHEADER));
	Quad2=Quad1+1;
	Quad3=Quad2+1;
	pDibBits=pMem+sizeof(BITMAPINFOHEADER)+3*sizeof(RGBQUAD);

	pBmpHeader->biSize=(DWORD)sizeof(BITMAPINFOHEADER);
	pBmpHeader->biWidth=(long)bmpX_pels;
	pBmpHeader->biHeight=(long)bmpY_pels;
	pBmpHeader->biPlanes=(WORD)1;
	pBmpHeader->biBitCount=(WORD)4;
	pBmpHeader->biCompression=(DWORD)BI_RGB;
	pBmpHeader->biSizeImage=(DWORD)0;
	pBmpHeader->biClrUsed=(DWORD)3;
	pBmpHeader->biClrImportant=(DWORD)0;

	Quad1->rgbBlue=0;
	Quad1->rgbGreen=0;
	Quad1->rgbRed=0;
	Quad1->rgbReserved=0;
	Quad2->rgbBlue=128;
	Quad2->rgbGreen=128;
	Quad2->rgbRed=128;
	Quad2->rgbReserved=0;
	Quad3->rgbBlue=255;
	Quad3->rgbGreen=255;
	Quad3->rgbRed=255;
	Quad3->rgbReserved=0;

	bOffSet=0;                             
	bTotal=(((long)bmpY_pels)*((long)(((bmpX_pels+15)/16)*16))*(*nbmps)/2);
	for(j=bmpY_pels;j>0;j--)
		{
		mask=0x01;
		low_nibble=FALSE;
		set_white=0x20;
		for(i=0;i<bmpX_pels;i++)
			{
 			if((*(pPic+((((long)(j-1))*(long)(((bmpX_pels+15)/16)*16)+i)/8)))&mask)
				(*(pDibBits+bOffSet))|=0;
			else
				(*(pDibBits+bOffSet))|=set_white;
			if(low_nibble)
				bOffSet++;
			low_nibble=!low_nibble;
			set_white^=0x22;
			mask=mask<<1;
			if(!mask)
				mask=0x01;
			if(pBmpHeader->biBitCount!=4)
				count=1;                                  
			}
		if(low_nibble)
			bOffSet++;
		if(bOffSet&1)
			bOffSet++;
		if(bOffSet&2)
			bOffSet+=2;
		if (statProc)
			{
			pct=(int)(bOffSet*100/bTotal);
			if ((*statProc)(pct))
				{
				GlobalUnlock(hPic);
				GlobalFree(hPic);
				GlobalUnlock(hMem);
		        *errorstate=5;
				return(NULL);
				}
			}
		}

	if(*nbmps==2)
		{
		bOffSet=0;
		pPic+=pWsHeader1->bitmap_bytes;
		for(j=bmpY_pels;j>0;j--)
			{
			mask=0x01;
			low_nibble=FALSE;
			for(i=0;i<bmpX_pels;i++)
				{
	 			if((*(pPic+((((long)(j-1))*(long)(((bmpX_pels+15)/16)*16)+i)/8)))&mask)
					if(low_nibble)
						{
						if((*(pDibBits+bOffSet)) & 0x0f)
							(*(pDibBits+bOffSet))^=0x03;
						}
					else
						if((*(pDibBits+bOffSet)) & 0xf0)
							(*(pDibBits+bOffSet))^=0x30;
				if(low_nibble)
					bOffSet++;
				low_nibble=!low_nibble;
				mask=mask<<1;
				if(!mask)
					mask=0x01;
				}
			if(low_nibble)
				bOffSet++;
			if(bOffSet&1)
				bOffSet++;
			if(bOffSet&2)
				bOffSet+=2;
			if (statProc)
				{
				pct=(int)(bOffSet*100/bTotal+50);
				if ((*statProc)(pct))
					{
					GlobalUnlock(hPic);
					GlobalFree(hPic);
					GlobalUnlock(hMem);
			        *errorstate=5;
					return(NULL);
					}
				}
			}
		}

	GlobalUnlock(hPic);
	GlobalFree(hPic);
	GlobalUnlock(hMem);
	return(hMem);
	}

BOOL SaveDib(LPCSTR szFileName,HGLOBAL hDib,LPINT errorstate)
	{
	BYTE __huge				*lpDib;
	HFILE					hFile;
	DWORD					nDibBytes;
	BITMAPFILEHEADER		bmfh;
	BITMAPINFOHEADER __huge	*bmih;

	if((hFile=_lcreat(szFileName,0))==HFILE_ERROR)
		{
        *errorstate=6;
		return(FALSE);
		}
	if(!(lpDib=(unsigned char __huge *)GlobalLock(hDib)))
		{
        *errorstate=7;
		_lclose(hFile);
		return(FALSE);
		}

    bmih=(LPBITMAPINFOHEADER)lpDib;
	nDibBytes=(((bmih->biWidth+31)/32)*32)*(bmih->biHeight)*(bmih->biBitCount)/8;
	bmfh.bfType=*(LPWORD) "BM";
	bmfh.bfSize=(DWORD)((DWORD)sizeof(BITMAPINFOHEADER)+(DWORD)(3*sizeof(RGBQUAD))+nDibBytes+(DWORD)sizeof(BITMAPFILEHEADER));
	bmfh.bfReserved1=0;
	bmfh.bfReserved2=0;
	bmfh.bfOffBits=(DWORD)sizeof(BITMAPINFOHEADER)+(DWORD)sizeof(BITMAPFILEHEADER)+(DWORD)(3*sizeof(RGBQUAD));
		
	if(HFILE_ERROR==_hwrite(hFile,(LPBYTE)&bmfh,sizeof(BITMAPFILEHEADER)))
		{
        *errorstate=8;
		_lclose(hFile);
		return(FALSE);
		}
	if(HFILE_ERROR==_hwrite(hFile,lpDib,(LONG)(bmfh.bfSize-(DWORD)sizeof(BITMAPFILEHEADER))))
		{
        *errorstate=8;
		_lclose(hFile);
		return(FALSE);
		}
	_lclose(hFile);
	GlobalUnlock(hDib);
	return(TRUE);
	}
