
/* 
Get and display all environment variables
Example taken from the Memory allocation chapter (Ch 7) of the
PLIB manual

INT p_findenviron(TEXT *pMatch, INT mLength, UBYTE *pBuf, HANDLE *pHandle);
Called repeatedly to find the name and value of all environment variables that 
match the wild card name pMatch of length mLength.
On the first call *pHandle should contain zero and subsequent calls pass the 
value that is written by the previous call. 
The function returns zero if a matching environment variable was found or the 
negative E_FILE_EOF when there are no more matching names. The wild card 
match string should remain the same on successive calls.
Each successful call writes the name and value to pBuf as two successive 
leading byte count strings giving the environment variable name followed by 
its value. The maximum length of an environment variable name is 
E_MAX_ENV_SIZE and the maximum length of a value is P_ENVMAX-1 bytes so 
pBuf should have room for E_MAX_ENV_SIZE+P_ENVMAX+1 bytes.
A wild card name of "*" will match all the environment variables.
In the following example, PrintAllEnv prints the name and 
(potentially binary) value of all the environment variables:
*/

/* #include <p_std.h> */
/* #include <epoc.h> */
#include <plib.h>

LOCAL_C VOID PrintData(UBYTE *p,UINT len)
    {
    UBYTE *pe=0;

    p_print("%d [",len);
    for (pe=p+len;p<pe;p++)
        p_isprint(*p) ? p_print("%c",*p) : p_print("<%02x>",*p);
    p_printf("]");
    }

GLDEF_C VOID PrintAllEnv(VOID)
    {
    UBYTE *p;
    
    HANDLE h;
    UBYTE b[E_MAX_ENV_SIZE+P_ENVMAX+1];

    for (h=0;p_findenviron("*",1,&b[0],&h)>=0;)
        {
        p=&b[0];
        PrintData(p+1,*p); /* print name */
        p+=*p+1;
        PrintData(p+1,*p); /* print value */
        }
    }

GLDEF_C INT main(VOID)
    {
      PrintAllEnv();
      p_getch();
      return(0);
    }
