/*	Application on		M:
	data on				B:
	index on			A:
	
*/

#include <plib.h>
#include <isam.g>

typedef struct
	{
	WORD				len;
	UBYTE				data[2];
	} ISAM_RECORDBUF;



LOCAL_D INT				ret;

LOCAL_D TEXT			DataFileName[P_FNAMESIZE];
LOCAL_D TEXT			IndexFileName[P_FNAMESIZE];

LOCAL_D VOID		*	pDebugFile;
LOCAL_D UINT			DebugFileMode;

LOCAL_D ULONG			Start;
LOCAL_D ULONG			End;
LOCAL_D P_DAYSEC		DaySecTime;
LOCAL_D P_DATE			DateTime;

LOCAL_D HANDLE			IsamCat;
LOCAL_D VOID		*	pIsamObject;

LOCAL_D ISAM_FIELDDEF	FieldDef;
LOCAL_D ISAM_KEYDEF		KeyDef;
LOCAL_D INT				IndexID;
LOCAL_D LONG			Count;
LOCAL_D ISAM_RECORDBUF	*	RecordBufAddress;


LOCAL_C VOID CloseDebugFile(VOID)
	{
	p_close(pDebugFile);
	}


LOCAL_C INT Check(INT val, TEXT *msg)
    {
    TEXT		buf[50];

    if (val >= 0)
        return val;
    p_atos(&buf[0],"Error : %s",msg);
    p_notifyerr(val,&buf[0],0,0,0);
	CloseDebugFile();
	p_exit(0);
    }


LOCAL_C VOID OpenDebugFile(TEXT *name)
	{
	DebugFileMode	=	P_FTEXT | P_FREPLACE | P_FUPDATE;

	Check(p_open(&pDebugFile, name, DebugFileMode), "opening debug file");
	}


LOCAL_C VOID DebugMess(TEXT *mess)
	{
	Check(p_write(pDebugFile, mess, p_slen(mess)), "writing debug file");
	}


LOCAL_C VOID LoadIsamDYL(VOID)
	{
	Check(p_loadlib("m:\\isam.dyl", &IsamCat, TRUE), "LoadIsamDYL: p_loadlib");
	}
	
	
LOCAL_C VOID CreateIsamObject(VOID)
	{
	pIsamObject	=	f_newlibhsend(IsamCat, C_BTDBF, O_IS_INIT, 4096, 0);
	}
	
	
LOCAL_C VOID SetFieldDefinition(VOID)
	{
	FieldDef.nFields	=	5;
	FieldDef.type[0]	=	ISAM_FIELDTYPE_STRING;
	FieldDef.type[1]	=	ISAM_FIELDTYPE_STRING;
	FieldDef.type[2]	=	ISAM_FIELDTYPE_STRING;
	FieldDef.type[3]	=	ISAM_FIELDTYPE_STRING;
	FieldDef.type[4]	=	ISAM_FIELDTYPE_DOUBLE;
	Check(ret =	p_entersend4(pIsamObject, O_IS_SET_FIELDDEF, 0, &FieldDef), "SetFieldDefinition: O_IS_SET_FIELDDEF");
	}

	
LOCAL_C VOID OpenDataFile(VOID)
	{
	Check(ret	=	p_entersend4(pIsamObject, O_IS_DOPEN, &DataFileName[0], P_FOPEN), "OpenDataFile: O_IS_DOPEN");
	}
	
	
LOCAL_C VOID SetIndexKeyDefinition(VOID)
	{
	KeyDef.nKeyFields	=	1;
	
	KeyDef.keyField[0].field   	=	3;
	KeyDef.keyField[0].type		=	ISAM_FIELDTYPE_STRING;
	KeyDef.keyField[0].flags   	=	ISAM_FIELDFLAG_ASCEND;
	KeyDef.keyField[0].offset	=	0;
	KeyDef.keyField[0].len		=	13;
	KeyDef.keyField[0].convert	=	ISAM_CONVERT_NOFOLD;
	Check(ret	=	p_entersend4(pIsamObject, O_IS_SET_KEYDEF, 0, &KeyDef), "SetIndexKeyDefinition: O_IS_SET_KEYDEF");
	}
	
	
LOCAL_C VOID CreateIndex(VOID)
	{
	Check(IndexID	=	p_entersend4(pIsamObject, O_IS_IOPEN, &IndexFileName[0], P_FCREATE | P_FUPDATE), "CreateIndex: O_IS_IOPEN");
	}
	
	
LOCAL_C VOID OpenIndex(VOID)
	{
	Check(IndexID	=	p_entersend4(pIsamObject, O_IS_IOPEN, &IndexFileName[0], P_FOPEN), "OpenIndex: O_IS_IOPEN");
	Check(p_entersend4(pIsamObject, O_IS_ICOUNT, IndexID, &Count), "OpenIndex: O_IS_ICOUNT");
	p_printf("Index contains %D entries", Count);
	}
	
	
LOCAL_C VOID BuildIndex(VOID)
	{
	Check(p_entersend3(pIsamObject, O_IS_IBUILD, IndexID), "BuildIndex: O_IS_IBUILD");	
	}
	
	
LOCAL_C VOID SetIndexFlags(VOID)
	{
	Check(p_entersend5(pIsamObject, O_IS_IFLAGS, IndexID, ISAM_FLAG_ALLOWDUP, ISAM_FLAG_ALLOWDUP), "SetIndexFlags: O_IS_IFLAGS");
	}


LOCAL_C INT LowFind(INT id, ...)
	{
	return(Check(ret	=	p_entersend4(pIsamObject, O_IS_IFIND, id, &id + 1), "LowFind: O_IS_IFIND"));
	}
	
	
LOCAL_C VOID FindRecordBuf(VOID)
	{
	RecordBufAddress	=	p_send2(pIsamObject, O_IS_GET_RBUF);
	}


LOCAL_C VOID FindRecord(VOID)
	{
	TEXT				SearchClue[15];
	
	
	SearchClue[0]	=	p_getl("Search clue ? :", &SearchClue[1], 13);
	
	
	Start	=	p_date();
	
	if (LowFind(IndexID, &SearchClue[0], NULL) == FALSE)
		p_printf("Found it !");
	else
		p_printf("Not there !");	
	
	End		=	p_date();
	p_getch();
	}
	
	
LOCAL_C VOID RecordTimes(VOID)
	{
	TEXT				StartBuf[30];
	TEXT				EndBuf[30];
	
	p_sttods(&Start, &DaySecTime);
	p_dstodt(&DaySecTime, &DateTime);
	p_atos(&StartBuf[0], "Find started at > %u:%u:%u",
								(UINT)DateTime.hour,
								(UINT)DateTime.minute,
								(UINT)DateTime.second);
								
	p_sttods(&End, &DaySecTime);
	p_dstodt(&DaySecTime, &DateTime);
	p_atos(&EndBuf[0],   "Find ended at   > %u:%u:%u",
								(UINT)DateTime.hour,
								(UINT)DateTime.minute,
								(UINT)DateTime.second);
								
	DebugMess(&StartBuf[0]);
	DebugMess(&EndBuf[0]);
	}
	
	
LOCAL_C VOID CleanUpAndQuit(VOID)
	{
	p_send2(pIsamObject, O_IS_DESTROY);
	CloseDebugFile();
	p_exit(0);
	}


LOCAL_C VOID Init(VOID)
	{
	p_scpy(&DataFileName[0], "b:\\data.dbf");
	p_scpy(&IndexFileName[0], "a:\\data.btx");
	}
	
	
GLREF_C VOID main(VOID)
	{
	OpenDebugFile("loc::m:\\debug.txt");
	DebugMess("About to init");
	Init();
	DebugMess("Done init");

	DebugMess("About to load DYL");
	LoadIsamDYL();
	DebugMess("Loaded DYL");

	DebugMess("About to create object");
	CreateIsamObject();
	DebugMess("Created object");

	DebugMess("About to set field definition");
	SetFieldDefinition();
	DebugMess("Set field definition");

	DebugMess("About to open data file");
	OpenDataFile();
	DebugMess("Opened data file");

	DebugMess("About to set key definition");
	SetIndexKeyDefinition();
	DebugMess("Set key definition");

	DebugMess("About to open index");
	OpenIndex();
	DebugMess("Opened index");

	DebugMess("About to set index flags");
	SetIndexFlags();
	DebugMess("Set index flags");

/*	DebugMess("About to build index");
	BuildIndex();
	DebugMess("Built index"); */

	DebugMess("About to find record");
	FindRecord();
	RecordTimes();
	FindRecordBuf();
	DebugMess("Done find + record times");

	DebugMess("About to clean up and quit");
	CleanUpAndQuit();
	}
