/*	WAV2WVE.CPP - Covert .WAV file to .WVE file
Written by MS, March 1993
Converted by NSM, April 1993
Copyright (C) Psion PLC 1993.

By the way, a SINT is a "signed int", etc.

 VER    DATE   PRG DESCRIPTION
===== ======== === ===========
1.00F -------- NSM First release
*/
#include <p_std.h>

#define ALawSignature "ALawSoundFile**"
#define SignatureSize 16

typedef struct
    {
    TEXT Signature[SignatureSize];
    UWORD Version;
    ULONG Samples;
    UWORD SilenceInTicks;
    UWORD Repeats;
    UWORD Spare[3];
    } SndFile;

#define ProgVersion "V1.20F (23/06/93)"

#define SIZE_BUF 0x2000
#define MAGICXOR 0x2a
#define ONE  1
#define BIAS 33
#define BIT4 0x10
#define BIT8 0x80 
#define BIT16 0x2000
#define MASK2 0x03
#define MASK3 0x70
#define MASK4 0x0f
#define MASK7 0x7f
#define MASK8 0xff

GLREF_D P_DEVICE p_file;

LOCAL_D	VOID *inFcb; // Input fcb
LOCAL_D	VOID *outFcb; // Out fcb
LOCAL_D SBYTE inBuf[SIZE_BUF+10];
LOCAL_D SBYTE outBuf[SIZE_BUF+10];
LOCAL_D ULONG inDataLen; // Length in bytes
LOCAL_D UINT sampleRate;
LOCAL_D UINT bytesPerSample;
LOCAL_D TEXT cmdBuf[P_CMDMAX]; // Command line buffer
LOCAL_D TEXT wavFileName[P_FNAMESIZE]; // .WAV file name
LOCAL_D TEXT wveFileName[P_FNAMESIZE]; // .WVE file name
LOCAL_D UINT Repeats; // Number of repeats
LOCAL_D UINT SilenceInTicks; // Silence in ticks
LOCAL_D UINT Brief; // Verbose display

LOCAL_C VOID parseCommandLine()
/*
Parse the command line arguments.
*/
    {

    TEXT *pInArg=NULL; // Initialise argument pointers
    P_GETFLG inArg;
    inArg.argm=1; // Set maximum number of each
    inArg.argp=(&pInArg);
    TEXT *pOArg=NULL; // Initialise argument pointers
    P_GETFLG oArg;
    oArg.argm=1; // Set maximum number of each
    oArg.argp=(&pOArg);
    if (p_getflg(&cmdBuf[0],"O*,R#,S#,B!",
    	&oArg,&Repeats,&SilenceInTicks,&Brief,&inArg) || !inArg.argc)
	{
	p_eprintf("Usage: [-Ofilename[.wve]] filename[.wav] [-Rrepeat -Ssilence -B]");
	p_eprintf("-Rrepeat      Set the number of repeats");
	p_eprintf("-Ssilence     Set the silence period in 1/32ths");
	p_eprintf("-B            Brief output");
	p_exit(TRUE);
	}
    if (Repeats==0)
	Repeats=1;
    SINT ret=p_fparse(pInArg,".WAV",&wavFileName[0],NULL);
    if (ret<0)
	uPanic("Parse .WAV file name",ret);
    if (oArg.argc)
	ret=p_fparse(pOArg,".WVE",&wveFileName[0],NULL);
    else
	ret=p_fparse(".WVE",&wavFileName[0],&wveFileName[0],NULL);
    if (ret<0)
	uPanic("Parse .WVE file name",ret);
    }

LOCAL_C VOID badFormat(SINT aVal)
/*
Display a bad file format number.
*/
    {

    p_eprintf("Bad file format [%d]",aVal);
    p_exit(TRUE);
    }

LOCAL_C VOID read(VOID *aPtr,SINT aLen)
/*
Read from the file.
*/
    {

    SINT ret=p_read(inFcb,aPtr,aLen);
    if (ret!=aLen)
	{
	if (ret>=0)
	    ret=E_FILE_EOF;
	uPanic("Reading .WAV file",ret);
	}
    }
LOCAL_C VOID relpos(SLONG pos)
/*
Do a relative pos 
*/
    {

    SLONG opos=0;
    p_seek(inFcb,P_FCUR,&opos);
    opos+=pos;
    SINT ret=p_seek(inFcb,P_FCUR,&pos);
    if (ret<0 || opos!=pos)
	{
	if (ret>=0)
	    ret=E_FILE_EOF;
	uPanic("Reading .WAV file",ret);
	}
    }

LOCAL_C VOID checkHeader(VOID)
    {

    SINT ret=p_open(&inFcb,&wavFileName[0],P_FOPEN);
    if (ret<0)
	uPanic("Opening .WAV file",ret);
    read(&inBuf[0],4);
    if (p_bcmp(&inBuf[0],4,"RIFF",4))
	badFormat(1);
    ULONG totLen;
    read(&totLen,4);
    read(&inBuf[0],4);
    if (p_bcmp(&inBuf[0],4,"WAVE",4))
	badFormat(2);
    FOREVER
	{	
	read(&inBuf[0],4);
	if (p_bcmp(&inBuf[0],4,"fmt ",4)==0)
	    break;
	SLONG pos;
	read(&pos,4);
	totLen=totLen-8-pos;
	relpos(pos);
	}
    ULONG locLen;
    read(&locLen,4);
    SINT i;
    read(&i,2);
    if (i!=1)
	{
	p_eprintf("Unexpected format tag: %d",i);
	p_exit(TRUE);
	}
    read(&i,2);
    if (i!=1)
	{
	p_eprintf("Unexpected channel: %d",i);
	p_exit(TRUE);
	}
    ULONG l;
    read(&l,4);
    sampleRate=(UINT)l;
    if (!(sampleRate==0xAC44 || sampleRate==22050 || sampleRate==11025))
	{
	p_eprintf("Unexpected sample rate: %u",sampleRate);
	p_exit(TRUE);
	}
    read(&l,4);
    read(&bytesPerSample,2);
    if (l!=bytesPerSample*sampleRate)
	uPanic("Strange sample rates");
    read(&i,2);
    if (i!=8*bytesPerSample)
	{
	p_eprintf("Unexpected data size: %d",i);
	p_exit(TRUE);
	}
    locLen-=0x10;
    if (locLen) // Skip extra header info
	read(&inBuf[0],(SINT)locLen);
    read(&inBuf[0],4);
    if (p_bcmp(&inBuf[0],4,"data",4))
	badFormat(3);
    read(&inDataLen,4);
    if ((totLen-locLen-0x24)!=((inDataLen+1)&0xfffffffel))
    	p_eprintf("Header lengths do not add up");
    }

GLDEF_C SINT aCompand(SINT x)
/*
Compress input 16-bit 2's complement integer (13-bit magnitude) to
8-bit signed compressed number using A-law.
*/
    {

    SINT p=BIT8;
    if (x&0x1000)
	{
	x= -x;
	x&=0xFFF;
	p=0;
	}
    SINT s;
    if (x&(MASK4<<8)) /* Find leading '1' using binary search */
	{
	if (x&(MASK2<<10))
	    s=(x&(ONE<<11)) ? 7 : 6;
	else
	    s=(x&(ONE<<9)) ? 5 : 4;
	}
    else
	{
	if (x&(MASK2<<6))
	    s=(x&(ONE<<7)) ? 3 : 2;
	else
	    s=(x&(ONE<<5)) ? 1 : 0;
	}
    SINT y;
    if (s==0)
	y=x>>1;
    else
	y=((x>>s)&MASK4)|(s<<4);
    return((~((y|p)^MAGICXOR))&MASK8);
    }

LOCAL_C VOID write(VOID *aPtr,SINT aLen)
/*
Write to the file.
*/
    {

    SINT ret=p_write(outFcb,aPtr,aLen);
    if (ret<0)
	uPanic("Writing .WVE file",ret);
    }

LOCAL_C VOID genOut(VOID)
    {

    SINT ret=p_open(&outFcb,&wveFileName[0],P_FREPLACE|P_FUPDATE);
    if (ret<0)
	uPanic("Creating .WVE file",ret);
    ULONG inLen=inDataLen;
    ULONG outLen=(inLen*8000.0)/sampleRate; // Floating point to allow for long files
    if (bytesPerSample==2)
    	outLen>>=1;
    SINT i=(SINT)(outLen>>10)+1;
    outLen=((ULONG)i)<<10;
    SBYTE *q=(&outBuf[0]);
    SLONG d=0l;
    SLONG con=(sampleRate>>3);
    SINT z=(SINT)(con/1000);
    SndFile head;
    p_zfil(&head,sizeof(head));
    p_scpy(&head.Signature[0],ALawSignature);
    head.Version=0x100F;
    head.Samples=outLen;
    head.SilenceInTicks=SilenceInTicks;
    head.Repeats=Repeats;
    write(&head,sizeof(head));
    if (bytesPerSample==1)
	{ // 8 bit samples
    	SBYTE *p=(&inBuf[0]);
	while (inLen)
	    {
	    i=(inLen>SIZE_BUF ? SIZE_BUF : (SINT)inLen);
	    read(p,i);
	    inLen-=i;
	    SBYTE *pE=p+i;
	    while (p<pE)
		{
		*p=(*p)-0x80; // Make it signed
		p++;
		}
    	    p=(&inBuf[0]);
	    while (p+z<pE)
		{
		d=1000-d;
		SLONG val=d*(*p++);
		d=con-d;
		SINT x=(SINT)(d/1000l);
		d-=1000l*(SLONG)x;
		for (SINT j=0;j<x;j++)
			val+=1000l*(*p++);
		val+=d*(*p);
		val*=16;
		val/=con;
		*q++=aCompand((SINT)val);
		}
	    UINT r=(UINT)(pE-p);
	    p_bcpy(&inBuf[0],p,r);
	    p=(&inBuf[r]);
	    r=(UINT)(q-&outBuf[0]);
	    write(&outBuf[0],r);
	    outLen-=r;
	    q=(&outBuf[0]);
	    }
	}
    else
	{ // 16 bit samples
    	SINT *p=(SINT *)(&inBuf[0]);
	while (inLen)
	    {
	    i=(inLen>SIZE_BUF ? SIZE_BUF : (SINT)inLen);
	    read(p,i);
	    inLen-=i;
	    SINT *pE=p+(i>>1);
//	    while (p<pE)
//		{
//		*p=(*p)-0x8000; // Make it signed
//		p++;
//		}
    	    p=(SINT *)(&inBuf[0]);
	    while (p+z<pE)
		{
		d=1000-d;
		SLONG val=d*(*p++);
		d=con-d;
		SINT x=(SINT)(d/1000l);
		d-=1000l*(SLONG)x;
		for (SINT j=0;j<x;j++)
			val+=1000l*(*p++);
		val+=d*(*p);
		val/=16; // Get into the right range
		val/=con;
		*q++=aCompand((SINT)val);
		}
	    UINT r=(UINT)(pE-p)<<1;
	    p_bcpy(&inBuf[0],p,r);
    	    p=(SINT *)(&inBuf[r]);
	    r=(UINT)(q-&outBuf[0]);
	    write(&outBuf[0],r);
	    outLen-=r;
	    q=(&outBuf[0]);
	    }
	}
    if (outLen>0) // Defensive
    	{
    	p_bfil(&outBuf[0],(SINT)outLen,0x55);
    	write(&outBuf[0],(SINT)outLen);
    	}
//
// Display header data
//
    ULONG tTime=(head.Samples+(head.SilenceInTicks*250))/80; // In 1/100ths of a second
    if (!Brief)
	{
	p_printf("Samples        = %ld(dec),%lx(hex) samples",head.Samples,head.Samples);
	p_printf("Silence        = %u(dec),%x(hex) 1/32ths of a second",head.SilenceInTicks,head.SilenceInTicks);
	p_printf("Silence at end = %ld(dec) samples, %ldms",outLen,outLen>>3);
	p_printf("Repeats        = %u(dec)",head.Repeats);
	p_printf("Single time    = %ld.%02u seconds",tTime/100,(UINT)(tTime%100));
	}
    tTime*=head.Repeats;
    p_printf("Total time     = %ld.%02u seconds",tTime/100,(UINT)(tTime%100));
    }

GLDEF_C SINT main(VOID)
/*
Convert a .WAV file to a .WVE file.
*/
    {

    p_inst(&p_file,NULL);
    inFcb=outFcb=NULL;
    p_eprintf("Wav2Wve %s Copyright (C) Psion PLC 1993",ProgVersion);
    p_getcom(&cmdBuf[0]); // Get the command line
    p_sconf(&cmdBuf[0]); // Convert to upper case
    parseCommandLine(); // Parse the command line
    p_printf("Converting %s to %s",&wavFileName[P_FSYSNAMESIZE],&wveFileName[P_FSYSNAMESIZE]);
    checkHeader();
    genOut();
    p_close(inFcb);
    p_close(outFcb);
    if (!Brief)
	p_eprintf("*** Coverted O.K. ***");
    return(FALSE);
    }

