#include <epoc.h>
#include <p_std.h>
#include <p_sys.h>
#include <p_file.h>
#include <p_xmodem.h>
#include <wlib.h>

#define  SEND_FILE1		"send1.bin"
#define  SEND_FILE2		"send2.bin"

LOCAL_D INT					ret;

LOCAL_D VOID			*	port;
LOCAL_D UWORD				type;
LOCAL_D UWORD 				mode;
LOCAL_D TEXT				buf[130];
LOCAL_D TEXT			*	p;
LOCAL_D UWORD				len;
LOCAL_D VOID			*	file;
LOCAL_D P_INFO				info;
LOCAL_D INT					BSent;


LOCAL_C INT Check(INT val, TEXT *msg)
    {
    TEXT		buf[50];

    if (val >= 0)
        return val;
    p_atos(&buf[0],"Error : %s: BSent %d", msg, BSent);
    p_notifyerr(val,&buf[0],0,0,0);
    p_exit(0);
    }


LOCAL_C VOID Open(VOID)
	{
	Check(p_open(&port, "TTY:A", -1), "opening the serial port");
	p_printf("Serial port opened OK");
	Check(p_open(&port, "XMD:", -1), "attaching the driver");
	p_printf("Driver attached OK");
	}


LOCAL_C VOID Connect(VOID)
	{
	type = P_XMDM_ACCP;  /*  We are sending files  */
	mode = P_YMODEM_MODE;  /*  YMODEM batch with 128-byte packets  */

	Check(p_iow(port, P_FCONNECT, &type, &mode), "P_FCONNECT");
	p_printf("Connect completed OK");
	p_printf("Connect mode was %u", mode);
	}
	
	
LOCAL_C VOID SendFile1(VOID)
	{
	/*  Build the first frame information  */

	p_bfil(&buf[0], 128, 0);  /*  Zero the buffer  */
	p_finfo(SEND_FILE1, &info);  /*  Get file info  */
	p = p_scpy(&buf[0], SEND_FILE1) + 1;  /*  Put in the file name  */
	p+=p_gltob(p, info.size, 10);  /*  Put in the size  */
	*p++=' ';  /*  Put in a space  */
	p+=p_gltob(p, info.modst, 8);  /*  Put in the date  */

	len = 128;  /*  Buffer is 128 bytes long  */

	Check(p_iow(port, P_FWRITE, &buf[0], &len), "writing the first frame");
	p_printf("First frame wrote OK");

	Check(p_open(&file, SEND_FILE1, P_FOPEN|P_FSTREAM|P_FSHARE), "opening the file");
	p_printf("File opened OK");

	FOREVER
		{
		len = Check(p_read(file, &buf[0], 128), "reading bytes from file");
		p_printf("Read %u bytes OK", len);
		BSent++;
		Check(p_iow(port, P_FWRITE, &buf[0], &len), "writing bytes to file");
		p_printf("Wrote %u bytes OK", len);
		if (len < 128)  /*  Last block  */
			break;
		}
	p_printf("Wrote all blocks OK");

	len = 0;  /* Force EOT to be sent */
	p_iow(port, P_FWRITE, &buf[0], &len);
	p_printf("Send EOT OK");
	
	p_close(file);
	p_printf("Closed file OK");
	p_getch();
	}


LOCAL_C VOID SendFile2(VOID)
	{
	/*  Build the first frame information  */

	p_bfil(&buf[0], 128, 0);  /*  Zero the buffer  */
	p_finfo(SEND_FILE2, &info);  /*  Get file info  */
	p = p_scpy(&buf[0], SEND_FILE2) + 1;  /*  Put in the file name  */
	p+=p_gltob(p, info.size, 10);  /*  Put in the size  */
	*p++=' ';  /*  Put in a space  */
	p+=p_gltob(p, info.modst, 8);  /*  Put in the date  */

	len = 128;  /*  Buffer is 128 bytes long  */

	Check(p_iow(port, P_FWRITE, &buf[0], &len), "writing the first frame");
	p_printf("First frame wrote OK");

	Check(p_open(&file, SEND_FILE2, P_FOPEN|P_FSTREAM|P_FSHARE), "opening the file");
	p_printf("File opened OK");

	FOREVER
		{
		len = Check(p_read(file, &buf[0], 128), "reading bytes from file");
		p_printf("Read %u bytes OK", len);
		BSent++;
		Check(p_iow(port, P_FWRITE, &buf[0], &len), "writing bytes to file");
		p_printf("Wrote %u bytes OK", len);
		if (len < 128)  /*  Last block  */
			break;
		}
	p_printf("Wrote all blocks OK");

	len = 0;  /* Force EOT to be sent */
	p_iow(port, P_FWRITE, &buf[0], &len);
	p_printf("Send EOT OK");
	
	p_close(file);
	p_printf("Closed file OK");
	p_getch();
	}


LOCAL_C VOID SendEndOfFileList(VOID)
	{
	/*  Build the empty frame information  */

	p_bfil(&buf[0], 128, 0);  /*  Zero the buffer  */
	len = 128;  /*  Buffer is 128 bytes long  */

	Check(p_iow(port, P_FWRITE, &buf[0], &len), "End of file list");
	p_printf("End of file list wrote OK");
	}



LOCAL_C VOID Disconnect(VOID)
	{
	Check(p_iow(port, P_FDISCONNECT), "disconnecting");
	}
	
	
LOCAL_C VOID CloseAndQuit(VOID)
	{
	p_close(port);  /*  Close channel to XMD driver  */
	p_printf("Closed XMD OK");
	p_close(port);  /*  Close channel to serial port  */
	p_printf("Closed TTY OK");
	p_printf("All done...");
	p_getch();
	p_exit(0);
	}

  
GLDEF_C VOID main(VOID)
	{
	Open();
	Connect();
	SendFile1();
	Disconnect();
	Connect();
	SendFile2();
	Disconnect();
	Connect();
	SendEndOfFileList();
	Disconnect();
	CloseAndQuit();
	}


