/*	PSIWINXL.H	-	PsiWin External Translator API Definition	*/

/*	Copyright DataViz, Inc, 1994 - All Rights Reserved.			*/

/*	Revision History

	09-09-94	Original - CAS

*/

#ifndef	_INC_PSIWINXL

#define	_INC_PSIWINXL

#ifdef __cplusplus
	extern "C" {
#endif

#include	<windows.h>

//	PSPD Flag definitions

#define		PATH_INSTALL			0x0001
#define		PATH_SELECT_APP			0x0002
#define		PATH_SELECT_FORMAT		0x0004
#define		PATH_IMPORT_REPLACE		0x0010
#define		PATH_IMPORT_APPEND	 	0x0020
#define		PATH_IMPORT_MERGE		0x0040
#define		PATH_EXPORT_REPLACE		0x0100
#define		PATH_EXPORT_APPEND		0x0200
#define		PATH_EXPORT_MERGE		0x0400
#define		PATH_SYNCHRONIZE		0x1000
#define		PATH_FROM_CLIPBOARD		0x0080
#define		PATH_TO_CLIPBOARD		0x0800
		
#define		PATH_TO_PSION			0x0070
#define		PATH_FROM_PSION			0x0700


typedef struct tagXMINFO
{
	HWND		hwndHost;
	HINSTANCE	hInstXM;
	LPSTR		pszHostName;

}	XMINFO, FAR *LPXMINFO;


typedef	BOOL	(WINAPI *XMSTATPROC)(int);

WORD WINAPI XLInstall(LPSTR lpPSPD);
/*
	This routine is called to conditionally install the translator.  It
	returns the pathFlags WORD for this translation path or zero
	if the translator is NOT to be installed.

	lpPSPD points to the resource that describes this translator.
*/

int WINAPI	XLSelectApp(LPSTR fileName);
/*
	This routine answers whether the indicated file is the application
	format exchanged by this translation path.

	Returns:

	0	The file is not this application format
	1	The file may be this application format
	2	The file is definitely this application format
	3	The file is definitely this application format, but is
		password protected
*/

int WINAPI	XLSelectFormat(LPSTR fileName);
/*
	This routine answers whether the indicated file is of the OEM 
	format exchanged by this translation path.

	Returns:

	-1	The file is this OEM format, but is password protected
	0	The file is not this OEM format
	1	The file is definitely this OEM format
*/

BOOL WINAPI	XLImport(LPXMINFO xminfo, LPSTR sourceFile, LPSTR resultFile, WORD xferMode,
						XMSTATPROC statusRtn);
/*
	This routine imports from the sourceFile to the indicated resultFile
	per the requested transfer mode.  The available transfer modes are

	1 - Overwrite
	2 - Append
	4 - Merge/Update
	8 - Synchronize

	Translation status is reported as a percent complete by calling the
	statusRtn.  A statusRtn return of TRUE indicates that the translation
	should be cancelled.

	Returns TRUE if the translation is sucessful, FALSE otherwise.
*/

BOOL WINAPI	XLExport(LPXMINFO xminfo, LPSTR sourceFile, LPSTR resultFile, WORD xferMode,
						XMSTATPROC statusRtn);
/*
	This routine exports from the sourceFile to the indicated resultFile
	per the requested transfer mode.  The available transfer modes are

	1 - Overwrite
	2 - Append
	4 - Merge/Update
	8 - Synchronize

	Translation status is reported as a percent complete by calling the
	statusRtn.  A statusRtn return of TRUE indicates that the translation
	should be cancelled.

	Returns TRUE if the translation is sucessful, FALSE otherwise.
*/

int WINAPI	XLGetLastErrorMsg(LPSTR msg, int nMsg);
/*
	This routine copies the error message string associated with the 
	last error to the indicated buffer transfering up to nMsg bytes
	including the trailing '\0' character.

	Returns the length of the string returned or 0 if there is no last
	error.
*/

BOOL WINAPI	_export	XLCopy2ClipBoard(LPXMINFO info, LPSTR sourceFile);
/*
  	This routine xlates the sourcefile into RTF (or WKS ) format and TEXT format and
  	then it copies them into CLIPBOARD.
  	
	Returns TRUE if the translation is sucessful, FALSE otherwise.
*/
                                                       
BOOL WINAPI	_export	XLPasteFromClipBoard(LPXMINFO info, LPSTR targetFile);
/* 
	This routine pastes either the RTF (or WKS) or the TEXT data format of the 
	CLIPBOARD into the targetFile.
	
	Returns TRUE if the translation is sucessful, FALSE otherwise.
  	
*/

int WINAPI _export LibMain(HANDLE hInstance, unsigned short wDataSeg,
					unsigned short cbHeapSize, LPSTR lpCmdLine);

typedef	WORD (WINAPI *XLINSTALLPROC)(LPSTR);
typedef int  (WINAPI *XLSELECTPROC)(LPSTR);
typedef	BOOL (WINAPI *XLXLATEPROC)(LPXMINFO, LPSTR, LPSTR, int, XMSTATPROC);
typedef	int  (WINAPI *XLERRORPROC)(LPSTR, int);
typedef	BOOL (WINAPI *XLCOPYCLPPROC)(LPXMINFO, LPSTR);
typedef BOOL (WINAPI *XLPASTECLPPROC)(LPXMINFO, LPSTR);

#ifdef __cplusplus
	}
#endif

#endif	//	_INC_PSIWINXL 


