/* SOLCM.C */

// This file includes methods for:

// SOLCM (subclass of COMMAN)
// SOLDLG (subclasses of DLGBOX, 5 dialogs in SOLIPEG)
// SOLEL

#include <solipeg.g>
#include <solipeg.rsg>
#include <hwim.h>

GLREF_D PR_SOLENG *             soleng ; // external ptr to engine
GLREF_D PR_APPMAN * const       w_am ; // ptr to application manager
GLREF_D PR_WSERV *              w_ws ; // ptr to window server client
GLREF_D PR_DLGBOX *             DatDialogPtr ; // ptr to dialog object
GLREF_D TEXT *                  DatApp1 ; // reserved application ptr

#define OPTIONS     (soleng->soleng.options)
#define HISTORY     (soleng->soleng.history)


// Launch a specific dialog

// class - the class of dialog
// resid - the resource id from the application resource file
// rbuf - additional data which needs to be passed into the dialog
LOCAL_C INT LaunchDialog(INT class, INT resid, VOID *rbuf) {
    DL_DATA     dld ;

    dld.id = resid ;
    dld.rbuf = rbuf ;
    dld.pdlg = NULL ;
    return(hLaunchDial(CAT_SOLIPEG_SOLIPEG, class, &dld)) ;
}


#pragma METHOD_CALL


/************************************/
/* Dialog Subclass Methods          */
/************************************/

// Initialization for Open/Save dialogs
METHOD VOID soldlg_open_dl_dyn_init(PR_DLGBOX *self) {
    // Set the filename to the one passed in the rbuf
    p_send4(    DatDialogPtr,
                O_WN_SET,
                1,
                ((SOLRBUF *) self->dlgbox.rbuf)->strFilename ) ;

    // Set the context sensitive help
    self->dlgbox.helprid = RH_LIBRARY ;
}

// Keypress handling for ENTER in Open/Save Dialog
METHOD INT soldlg_open_dl_key(  PR_DLGBOX *     self,
                                INT             index,
                                INT             keycode,
                                INT             actbut ) {
    // Set the name in rbuf to that accepted in dialog
    p_send4(    DatDialogPtr,
                O_WN_SENSE,
                1,
                ((SOLRBUF *) self->dlgbox.rbuf)->strFilename ) ;
    // Change accepted
    return(WN_KEY_CHANGED) ;
}

// Initialization for Options dialog
METHOD VOID soldlg_options_dl_dyn_init(PR_DLGBOX *self) {
    INT     iLp ;

    // Set the dialog choices as currently recorded in the engine
    for ( iLp = 0 ; iLp < 5 ; iLp++ ) {
        hDlgSetChlist(iLp + 1, *(&OPTIONS.shape + iLp)) ;
    }

    // Set the context-sensitive help
    self->dlgbox.helprid = RH_OPTIONS ;
}

// Keypress handling for ENTER in Options dialog
METHOD INT soldlg_options_dl_key(   PR_DLGBOX *     self,
                                    INT             index,
                                    INT             keycode,
                                    INT             actbut ) {
    INT     iLp ;

    // Set the dialog choices into the engine settings
    for ( iLp = 0 ; iLp < 5 ; iLp++ ) {
        *(&OPTIONS.shape + iLp) = hDlgSenseChlist(iLp + 1) ;
    }

    // Change accepted
    return(WN_KEY_CHANGED) ;
}

// Initialization for Library dialog
METHOD VOID soldlg_library_dl_dyn_init(PR_DLGBOX *self) {
    // Set the dialog choices as currently recorded in the engine
    hDlgSetChlist(1, soleng->soleng.ridDefault) ;

    // Set the context-sensitive help
    self->dlgbox.helprid = RH_LIBRARY ;
}

// Keypress handling for ENTER in Library dialog
METHOD INT soldlg_library_dl_key(   PR_DLGBOX *     self,
                                    INT             index,
                                    INT             keycode,
                                    INT             actbut ) {
    // Set the dialog choices into the engine settings
    soleng->soleng.ridDefault = hDlgSenseChlist(1) ;

    // Change accepted
    return(WN_KEY_CHANGED) ;
}

// External library choice list dynamic initialization
METHOD VOID soldlg_ext_lib_dl_dyn_init(PR_DLGBOX * self) {
    TEXT            buffer[P_FNAMESIZE] ;
    SE_CHLIST       sc ;

    // Read the external library's name
    p_send4(    ((PR_SOLEL *) self->dlgbox.rbuf)->solel.pRsc,
                O_RS_READ_BUF,
                3,
                buffer ) ;
    if ( buffer[0] ) {
        hDlgSetText(0, buffer) ;
    }
    sc.set_flags = SE_CHLIST_DATA | SE_CHLIST_RETAIN ;

    // Set choice list to the list loaded from the library by solel
    sc.data = ((PR_SOLEL *) self->dlgbox.rbuf)->solel.pData ;
    hDlgSet(1, &sc) ;

    // Set the context-sensitive help
    self->dlgbox.helprid = RH_LIBRARY ;
}

// External library choice list accept
METHOD INT soldlg_ext_lib_dl_key(   PR_DLGBOX *     self,
                                    INT             index,
                                    INT             keycode,
                                    INT             actbut ) {
    // Tell the engine to load the resource chosen
    p_send4(    soleng,
                O_ENG_EXTERNAL_LIBRARY,
                self->dlgbox.rbuf,
                hDlgSenseChlist(1) ) ;

    // Change accepted
    return(WN_KEY_CHANGED) ;
}

// Statistics dialog dynamic initialization
METHOD VOID soldlg_stats_dl_dyn_init(PR_DLGBOX * self) {
    TEXT            buffer[32] ;
    INT             iLp ;
    INT             fFirst = TRUE ;
    TEXT *          p ;

    // Display statistics from the engine (persistently stored in an
    //  environment variable (DatApp1 stores the name)
    buffer[p_gltob(buffer, HISTORY.games, 10)] = '\x0' ;
    hDlgSetText(1, buffer) ;
    if ( HISTORY.games ) {
        buffer[p_gltob(buffer, HISTORY.pegs / HISTORY.games, 10)] = '\x0' ;
        hDlgSetText(2, buffer) ;
    }

    for ( buffer[0] = '\x0', p = &buffer[0], iLp = 0 ; iLp < 5 ; iLp++ ) {
        if ( HISTORY.results[iLp] ) {
            if ( !fFirst ) {
                *p++ = ',' ;
                *p++ = ' ' ;
            }
            fFirst = FALSE ;
            p += p_gtob(p, HISTORY.results[iLp], 10) ;
            *p = '\x0' ;
        }
    }
    hDlgSetText(3, buffer) ;

    // Set the context-sensitive help
    self->dlgbox.helprid = RH_MAIN ;
}

/************************************/
/* Command Manager Subclass Methods */
/************************************/

// Handle the exit - saving the board position to env
METHOD VOID solcm_com_exit(PR_SOLCM *self) {
    // Store the options etc. in an environment variable
    p_setenviron(   DatApp1,
                    p_slen(DatApp1),
                    &OPTIONS,
                    sizeof(OPTIONS) + sizeof(HISTORY) ) ;
    p_exit(0) ;
}

// Handle the mode change due to diamond key
METHOD VOID solcm_com_mode_change(PR_SOLCM *self, INT back) {
    // Play<->Edit mode change
    p_send3(soleng, O_ENG_MODE_CHANGE, back) ;
}

// New game command
METHOD VOID solcm_scm_new(PR_SOLCM *self) {
    // Confirm New Game request
    if ( hConfirm(RS_NEW_GAME) ) {
        p_send2(soleng, O_ENG_NEW_GAME) ;
    }
}

// Open game command
METHOD VOID solcm_scm_open(PR_SOLCM *self, INT comid) {
    SOLRBUF     rb ; // structure to be passed into file dialogs
    INT         iResID ;

    hLoadResBuf(    comid == O_SCM_EXTERNAL ? RS_DEF_EXTERNAL : RS_DEFNAME,
                    rb.strFilename ) ;

    // Choose which dialog
    switch ( comid ) {
        case O_SCM_OPEN :
            iResID = OPEN_DIALOG ;
            break ;

        case O_SCM_SAVE :
            iResID = SAVE_DIALOG ;
            break ;

        case O_SCM_EXTERNAL :
            iResID = OPEN_EXTERNAL_DIALOG ;
            break ;

        default :
            return ;
    }

    // Launch the dialog
    if ( !LaunchDialog(C_SOLDLG_OPEN, iResID, &rb) ) {
        // dialog was cancelled
        return ;
    }

    // If an external library is accepted
    if ( comid == O_SCM_EXTERNAL ) {
        PR_SOLEL *      pEL ;

        // Create a SOLEL object
        pEL = p_new(CAT_SOLIPEG_SOLIPEG, C_SOLEL) ;

        // Initialize it - this loads the list
        if ( p_send5(   pEL,
                        O_EL_INIT,
                        rb.strFilename,
                        self->solcm.strLibID,
                        self->solcm.wMaxLibVer ) ) {
            // Problem
            p_send5(w_am, O_AM_NOTIFY, RS_BADFILE, NULL, NULL) ;
        }
        else {
            // Get a choice of problem out of the library
            LaunchDialog(C_SOLDLG_EXT_LIB, EXTERNAL_LIBRARY_DIALOG, pEL) ;
        }

        // No more need for the SOLEL object
        hDestroy(pEL) ;
    }
    else {
        // Open or save a single game
        p_send3(    soleng,
                    comid == O_SCM_OPEN ? O_ENG_OPEN_GAME : O_ENG_SAVE_GAME,
                    rb.strFilename) ;
        p_send2(soleng, O_ENG_UPDATE_BOARD) ;
    }
}

// Use library command
METHOD VOID solcm_scm_library(PR_SOLCM *self) {
    // Internal library - simple application resource load
    LaunchDialog(C_SOLDLG_LIBRARY, LIBRARY_DIALOG, NULL) ;

    // Start a new game
    p_send2(soleng, O_ENG_NEW_GAME) ;
}

// Launch the Options dialog and reset board as necessary
METHOD VOID solcm_scm_options(PR_SOLCM *self) {
    UWORD   uwOldShape ;
    UWORD   uwOldRules ;
    UWORD   uwOldShow ;

    // Get current settings
    uwOldShape = OPTIONS.shape ;
    uwOldRules = OPTIONS.rules ;
    uwOldShow = OPTIONS.show ;

    LaunchDialog(C_SOLDLG_OPTIONS, OPTIONS_DIALOG, NULL) ;

    // Certain option changes require more work then others
    if ( OPTIONS.shape != uwOldShape ) {
        // Only shape changed
        p_send2(soleng, O_ENG_SHAPE) ;
    }
    else if ( OPTIONS.rules != uwOldRules ) {
        // Rules changed - need to recalculate and redraw movable pieces
        p_send2(soleng, O_ENG_UPDATE_BOARD) ;
    }
    else if ( OPTIONS.show != uwOldShow ) {
        // Board needs to be redrawn
        wInvalidateWin(w_ws->wserv.cli->win.id) ;
    }

    // Reposition window regardless
    p_send3(w_ws->wserv.cli, O_WN_REPOS, OPTIONS.winpos) ;
}

// Use statistics command
METHOD VOID solcm_scm_statistics(PR_SOLCM *self) {
    LaunchDialog(C_SOLDLG_STATS, STATS_DIALOG, NULL) ;
}

// Solipeg Command Manager Initialization
METHOD VOID solcm_com_init(PR_SOLCM *self) {
    hLoadResource(RS_LIBID, &self->solcm.strLibID) ;
    hLoadResBuf(RS_MAXLIBVER, &self->solcm.wMaxLibVer) ;
}

/************************************/
/* External Library Class Methods   */
/************************************/

// Initialize an external library for loading
METHOD INT solel_el_init(   PR_SOLEL *  self,
                            TEXT *      strFilename,
                            TEXT *      strReqID,
                            WORD        wMaxLibVer ) {
    TEXT *      strLibID = NULL ;
    INT         iRet ;
    INT         iLp ;
    TEXT        buf[33] ;

    // Create an intance of the rscfile class
    self->solel.pRsc = p_new(CAT_SOLIPEG_OLIB, C_RSCFILE) ;
    p_send3(self->solel.pRsc, O_RS_INIT, strFilename) ;

    // Check the library ID - should match strReqID
    p_send4(self->solel.pRsc, O_RS_READ, 1, &strLibID) ;
    iRet = p_scmp(strLibID, strReqID) ;
    p_free(strLibID) ;
    if (iRet)
        return(1) ;

    // Read the version number
    p_send4(self->solel.pRsc, O_RS_READ_BUF, 2, &self->solel.wLibVer) ;
    if ( self->solel.wLibVer > wMaxLibVer )
        return(1) ;

    // Read the library size
    p_send4(self->solel.pRsc, O_RS_READ_BUF, 4, &self->solel.bLibSize) ;

    // Create a container for the names
    self->solel.pData = p_new(CAT_SOLIPEG_OLIB, C_VASTR) ;
    p_send3(self->solel.pData, O_VA_INIT, 32) ;

    // Start after the header and load all the names
    for ( iLp = 5 ; iLp < 5 + self->solel.bLibSize ; iLp++ ) {
        // Read the name into a local buffer
        p_send4(    self->solel.pRsc,
                    O_RS_READ_BUF,
                    iLp,
                    buf ) ;
        // Insert it into the list
        p_send5(    self->solel.pData,
                    O_VA_INSERTM,
                    self->solel.pData->varoot.nrec,
                    buf,
                    1 ) ;
    }

    return(0) ;
}

// Read a game by number
METHOD VOID solel_el_read(  PR_SOLEL *  self,
                            VOID **     ppBuf,
                            INT         iIndex ) {
    // Just a simple shell around the resource read service
    // Note the use of bLibSize member
    p_send4(    self->solel.pRsc,
                O_RS_READ,
                5 + self->solel.bLibSize + iIndex,
                ppBuf ) ;
}

