// SOLENG.C - Solipeg engine

// This file includes methods for:

// SOLENG - the SOLIPEG engine

#include <plib.h>
#include <solipeg.g>
#include <solipeg.rsg>
#include <hwim.h>
#include <solglob.h>


// Table of direction vectors
GLDEF_D VECTOR          vectors[N_VECTORS] = {  {-1,  0},
						{-1, -1},
						{ 0, -1},
						{ 1, -1},
						{ 1,  0},
						{ 1,  1},
						{ 0,  1},
						{-1,  1} } ;

GLDEF_D P_RECT          rectPieces[] =  {   {   {0, 0},
						{DESIGN_SIZE, DESIGN_SIZE} },
					    {   {DESIGN_SIZE - 2, 0},
						{DESIGN_SIZE - 2 + DESIGN_SIZE, DESIGN_SIZE} },
					    {   {2 * (DESIGN_SIZE - 2), 0},
						{2 * (DESIGN_SIZE - 2) + DESIGN_SIZE, DESIGN_SIZE} }
					} ;

GLDEF_D BYTE    minY[] = { 2, 1, 0, 0, 0, 1, 2 } ;

// lValue MACROS - these shorthand macros are real useful
#define BOARD               (self->soleng.board)
#define PFCB                (self->soleng.pfcb)
#define this                (self->soleng)

#define INBOUNDS(x,y)       (   (x) >= 0 && (x) < BOARD_SIZE_X &&\
				(y) >= 0 && (y) < BOARD_SIZE_Y )
#define USABLE(x,y)         ( INBOUNDS((x), (y)) && BOARD[(x)][(y)] >= S_EMPTY )
#define EMPTY(x,y)          ( INBOUNDS((x), (y)) && BOARD[(x)][(y)] == S_EMPTY )
#define FULL(x,y)           ( INBOUNDS((x), (y)) && BOARD[(x)][(y)] >= S_FULL )


#pragma METHOD_CALL


// Draw the board in the gurrent GC (changing as necessary)
METHOD VOID soleng_eng_draw_board(PR_SOLENG *self, PR_SOLWIN *pWin) {
    G_GC        gc ;
    BYTE        x ;
    BYTE        y ;
    P_RECT      rect ;
    P_POINT     pos ;
    P_RECT *    prect[3] ;

    prect[0] = &rectPieces[0] ;
    prect[1] = &rectPieces[1] ;
    prect[2] = &rectPieces[options.show ? 2 : 1] ;

    gc.flags = 0 ;
    gSetGC(0, G_GC_MASK_GREY, &gc) ;

    for (   pos.x = 0, x = 0 ;
	    x < BOARD_SIZE_X ;
	    x++, pos.x += DESIGN_OFFSET ) {
	for (   y = minY[x] ;
		y < BOARD_SIZE_Y - minY[x] ;
		y++ ) {
	    if ( BOARD[x][y] != S_UNUSABLE ) {
		pos.y = y * DESIGN_OFFSET ;
		gCopyBit(   &pos,
			    pWin->solwin.hBMP,
			    prect[BOARD[x][y] - S_EMPTY],
			    G_TRMODE_REPL ) ;
	    }
	}
    }

    gc.flags = G_GC_FLAG_GREY_PLANE ;
    gSetGC(0, G_GC_MASK_GREY, &gc) ;

    for (   pos.x = 0, x = 0 ;
	    x < BOARD_SIZE_X ;
	    x++, pos.x += DESIGN_OFFSET ) {
	for (   y = minY[x] ;
		y < BOARD_SIZE_Y - minY[x] ;
		y++ ) {
	    if ( BOARD[x][y] != S_UNUSABLE ) {
		pos.y = y * DESIGN_OFFSET ;
		gCopyBit(   &pos,
			    pWin->solwin.hBMPGrey,
			    prect[BOARD[x][y] - S_EMPTY],
			    G_TRMODE_REPL ) ;
	    }
	}
    }

    gc.flags = G_GC_FLAG_BOTH_PLANES ;
    gSetGC(0, G_GC_MASK_GREY, &gc) ;

    for (   rect.tl.x = 0, rect.br.x = DESIGN_SIZE, x = 0 ;
	    x < BOARD_SIZE_X ;
	    x++, rect.tl.x += DESIGN_OFFSET, rect.br.x += DESIGN_OFFSET ) {
	for (   rect.tl.y = 0, rect.br.y = DESIGN_SIZE, y = 0 ;
		y < BOARD_SIZE_Y ;
		y++, rect.tl.y += DESIGN_OFFSET, rect.br.y += DESIGN_OFFSET ) {
	    if ( BOARD[x][y] == S_UNUSABLE ) {
		/* Clear */
		gClrRect(&rect, G_TRMODE_CLR) ;
	    }
	}
    }

}

METHOD INT soleng_eng_moves(PR_SOLENG *self) {
    BYTE    x ;
    BYTE    y ;
    BYTE    iLp ;
    INT     iIncrement ;

    iIncrement = options.rules ? 1 : 2 ;

	for ( x = 0 ; x < BOARD_SIZE_X ; x++ ) {
	for ( y = minY[x] ; y < BOARD_SIZE_Y - minY[x] ; y++ ) {
	    if ( BOARD[x][y] >= S_FULL ) {
		for ( iLp = 0 ; iLp < N_VECTORS ; iLp += iIncrement ) {
					if (    FULL(x + vectors[iLp].x, y + vectors[iLp].y) &&
			    EMPTY(x + 2 * vectors[iLp].x, y + 2 * vectors[iLp].y) ) {
			return(FALSE) ;
					}
				}
			}
	}
    }

    return(TRUE) ;
}

// Update (internal and graphical) board to reflect changes due to movement
METHOD VOID soleng_eng_update_board(PR_SOLENG *self) {
    if ( options.show ) {
	BYTE    x ;
	BYTE    y ;
	BYTE    iLp ;
	INT     iIncrement ;

	iIncrement = options.rules ? 1 : 2 ;

	for ( x = 0 ; x < BOARD_SIZE_X ; x++ ) {
	    for ( y = minY[x] ; y < BOARD_SIZE_Y - minY[x] ; y++ ) {
		if ( BOARD[x][y] >= S_FULL ) {
		    BOARD[x][y] = S_FULL ;
		    for ( iLp = 0 ; iLp < N_VECTORS ; iLp += iIncrement ) {
			if (    FULL(x + vectors[iLp].x, y + vectors[iLp].y) &&
				EMPTY(x + 2 * vectors[iLp].x, y + 2 * vectors[iLp].y) ) {
			    BOARD[x][y] = S_MOVABLE ;
			    break ;
			}
		    }
		}
	    }
	}
    }
}

// Reset the game according to the default starting position
METHOD VOID soleng_eng_init(PR_SOLENG *self) {
    VOID *      pbuf = NULL ;

    hLoadResource(SG_PL + options.shape, &pbuf) ;
    p_bcpy(this.board, pbuf, sizeof(this.board)) ;
    p_free(pbuf) ;
    p_send2(self, O_ENG_SAVE_BOARD) ;
}

// Reset the game according to the default starting position
METHOD VOID soleng_eng_new_game(PR_SOLENG *self) {
    p_bcpy(this.board, this.new_board, sizeof(this.new_board)) ;

    p_send3(w_ws->wserv.com, O_SCM_SPOS, XY(3, 5)) ;
    p_send2(w_ws->wserv.cli, O_WN_CURSOR) ;
    p_send2(w_ws->wserv.com, O_SCM_UPDATE) ;
}

// Reset the game according to the default starting position
METHOD VOID soleng_eng_save_board(PR_SOLENG *self) {
    p_bcpy(this.new_board, this.board, sizeof(this.board)) ;
}

// Read in a new game from the specified file
METHOD VOID soleng_eng_open_game(PR_SOLENG *self, TEXT *strFilename) {
    BYTE    buf[sizeof(BOARD)] ;
    INT     iRet ;

    if ( PFCB != NULL ) {
	p_close(PFCB) ;
	PFCB = NULL ;
    }

    // Load the game string
    f_open(&PFCB, strFilename, P_FSTREAM | P_FOPEN) ;
    iRet = f_read(PFCB, buf, sizeof(buf)) ;
    p_close(PFCB) ;
    PFCB = NULL ;

    if ( iRet != sizeof(BOARD) ) {
	// Some sort of error
	p_send5(w_am, O_AM_NOTIFY, RS_BADFILE, NULL, NULL) ;
	f_leave(E_GEN_FAIL) ;
    }

    for (   iRet = 0 ;
	    iRet < sizeof(BOARD) &&
		buf[iRet] >= S_UNUSABLE &&
		buf[iRet] <= S_MOVABLE ;
	    iRet++ ) ;
    if ( iRet < sizeof(BOARD) ) {
	// Some bounds error
	p_send5(w_am, O_AM_NOTIFY, RS_BADFILE, NULL, NULL) ;
	f_leave(E_GEN_FAIL) ;
    }

    // Must be okay, use it
    p_bcpy(BOARD, buf, sizeof(BOARD)) ;
    p_send2(self, O_ENG_SAVE_BOARD) ;

    // Set SHAPE: if it looks French or not
    options.shape = USABLE(1, 1) ;
}

// Save a game to the specified file
METHOD VOID soleng_eng_save_game(PR_SOLENG *self, TEXT *strFilename) {
    if ( PFCB != NULL ) {
	p_close(PFCB) ;
	PFCB = NULL ;
    }

    hEnsurePath(strFilename) ;
    f_open(&PFCB, strFilename, P_FSTREAM | P_FUPDATE | P_FREPLACE) ;
    f_write(PFCB, BOARD, sizeof(BOARD)) ;
    p_close(PFCB) ;
    PFCB = NULL ;
    p_send2(self, O_ENG_SAVE_BOARD) ;
}

METHOD VOID soleng_eng_internal_library(    PR_SOLENG * self,
					    INT         rid ) {
    VOID *      pbuf = NULL ;

    hLoadResource(SG_PL + rid, &pbuf) ;
    p_bcpy(this.board, pbuf, sizeof(this.board)) ;
    p_free(pbuf) ;
    p_send2(self, O_ENG_SAVE_BOARD) ;
}

// Load a game from an external library
METHOD VOID soleng_eng_external_library(    PR_SOLENG * self,
					    PR_SOLEL *  pEL,
					    INT         iIndex ) {
    TEXT *      pbuf = NULL ;
    INT         iLp ;

    // Call the solel object we were passed and get the string
    p_send4(pEL, O_EL_READ, &pbuf, iIndex) ;
    for (   iLp = 0 ;
	    iLp < sizeof(BOARD) &&
		pbuf[iLp] >= S_UNUSABLE &&
		pbuf[iLp] <= S_MOVABLE ;
	    iLp++ ) ;
    if ( iLp >= sizeof(BOARD) ) {
	// OK
	p_bcpy(BOARD, pbuf, sizeof(BOARD)) ;
    }
    else {
	// Some sort of bounds error
	p_send5(w_am, O_AM_NOTIFY, RS_BADFILE, NULL, NULL) ;
    }
    p_free(pbuf) ;
    p_send2(self, O_ENG_SAVE_BOARD) ;

    // Update board
    p_send3(w_ws->wserv.com, O_SCM_SPOS, XY(3, 5)) ;
    p_send2(w_ws->wserv.cli, O_WN_CURSOR) ;
    p_send2(w_ws->wserv.com, O_SCM_UPDATE) ;
}

// Update history for win
METHOD INT soleng_eng_endgame(PR_SOLENG *self) {
    INT     x ;
    INT     y ;

    // Shift the results down
    p_bcpy( &options.results[1],
	    &options.results[0],
	    sizeof(options.results) - sizeof(options.results[0])) ;

    options.results[0] = 0 ;

    // Add up the pegs left in this game
    for ( x = 0 ; x < BOARD_SIZE_X ; x++ ) {
	for ( y = minY[x] ; y < BOARD_SIZE_Y - minY[x] ; y++ ) {
	    options.results[0] += FULL(x, y) ;
	}
    }

    // Update total pegs left
    options.pegs += options.results[0] ;

    // One more game played
    options.games++ ;

    // Returns TRUE if the game was truly won by getting the last peg in
    // the center hole
    return( options.results[0] == 1 && FULL(3, 3) ) ;
}

METHOD VOID soleng_eng_toggle(PR_SOLENG *self, UWORD xy) {
    if ( USABLE(X(xy), Y(xy)) ) {
	BOARD[X(xy)][Y(xy)] = (BOARD[X(xy)][Y(xy)] == S_EMPTY ? S_FULL : S_EMPTY) ;
	p_send2(w_ws->wserv.com, O_SCM_UPDATE) ;
    }
}

METHOD INT soleng_eng_usable(   PR_SOLENG * self,
				UWORD       xy ) {
    return(USABLE(X(xy), Y(xy))) ;
}

METHOD INT soleng_eng_jump_valid(   PR_SOLENG * self,
				    INT         x,
				    INT         y,
				    INT         iDir ) {
    return( FULL(x, y) &&
	    FULL(x + vectors[iDir].x, y + vectors[iDir].y) &&
	    EMPTY(x + 2 * vectors[iDir].x, y + 2 * vectors[iDir].y) ) ;
}

METHOD VOID soleng_eng_jump(    PR_SOLENG * self,
				INT         x,
				INT         y,
				INT         iDir ) {
    BOARD[x][y] = S_EMPTY ;
    BOARD[x + vectors[iDir].x][y + vectors[iDir].y] = S_EMPTY ;
    BOARD[x + 2 * vectors[iDir].x][y + 2 * vectors[iDir].y] = S_FULL ;

    p_send3(w_ws->wserv.com, O_SCM_SPOS, XY(x + 2 * vectors[iDir].x, y + 2 * vectors[iDir].y)) ;
    p_send2(w_ws->wserv.cli, O_WN_CURSOR) ;
    p_send2(w_ws->wserv.com, O_SCM_UPDATE) ;
}

METHOD VOID soleng_eng_unjump(  PR_SOLENG * self,
				INT         x,
				INT         y,
				INT         iDir ) {
    BOARD[x][y] = S_FULL ;
    BOARD[x + vectors[iDir].x][y + vectors[iDir].y] = S_FULL ;
    BOARD[x + 2 * vectors[iDir].x][y + 2 * vectors[iDir].y] = S_EMPTY ;

    p_send3(w_ws->wserv.com, O_SCM_SPOS, XY(x, y)) ;
    p_send2(w_ws->wserv.cli, O_WN_CURSOR) ;
    p_send2(w_ws->wserv.com, O_SCM_UPDATE) ;
}

