// SOLMAIN.C

// This file includes methods for:

// SOLAM (subclass of HWIMMAN)
// SOLWS (subclass of WSERV)

#include <solipeg.g>
#include <solipeg.rsg>
#include <hwim.h>
#include <epoc.h>
#include <p_gen.h>
#include <solglob.h>
#include <slresver.h>

GLDEF_D PR_SOLENG *             soleng ;    // Ptr to engine object
GLDEF_D OPTIONS_tag             options ;


/************************************/
/* Solipeg Main Entry Point         */
/************************************/

GLDEF_C VOID main(VOID) {
    // Structures containing initializing data
    //  used in creating hwimman and wserv instances
    IN_HWIMMAN          app ;
    IN_WSERV            wserv ;

    // Dynamically link HWIM and OLIB categories
    p_linklib(0) ;

    // We'll be using our application resource file, the system resource
    //  file, the built-in cleanup functionality, and full S3a screen
    app.flags = FLG_APPMAN_RSCFILE |
                    FLG_APPMAN_SRSCFILE |
                    FLG_APPMAN_CLEAN |
                    FLG_APPMAN_FULLSCREEN ;

    // The category for all my classes is SOLIPEG
    wserv.com_cat = app.wserv_cat = p_getlibh(CAT_SOLIPEG_SOLIPEG) ;

    // My window server class is customized: C_SOLWS
    app.wserv_class = C_SOLWS ;

    // My command manager class is customized: C_SOLCM
    wserv.com_class = C_SOLCM ;

    // Create a new C_SOLAM object
    //  p_new() returns a ptr to the new solam object, which is
    //  sent the initialization method using app for initial
    //  appman/hwimman settings, and wserv for initial wserv settings.
    //  solam will create an instance of both solws and solcm according
    //  to these parameters
    p_send4(p_new(CAT_SOLIPEG_SOLIPEG, C_SOLAM), O_AM_INIT, &app, &wserv) ;
}


#pragma METHOD_CALL


/******************************************/
/* Application Manager Subclass Method    */
/******************************************/

// Find the application's resource file
METHOD VOID solam_am_rscname(PR_SOLAM *self, TEXT *strName) {
    P_INFO      info ;

    // Build a suitable name string - possible machine dependent
    p_atos(strName, "\\APP\\SOLIPEG\\SOLI%02d.RZC", p_getlanguage()) ;

    // parse the name with respect to the executable's location
    f_fparse(strName, DatCommandPtr, strName, NULL) ;

    // if the file doesn't exist, call the superclass' original method
    if ( p_finfo(strName, &info) < 0 ) {
        p_supersend3(self, O_AM_RSCNAME, strName) ;
    }
}

/************************************/
/* Window Server Subclass Methods   */
/************************************/

// Initialize the window server client
METHOD VOID solws_ws_dyn_init(PR_SOLWS *self) {
    hLoadResource(RS_RESOURCE_VER, &DatApp1) ;
    if ( p_scmp(DatApp1, SOLIPEG_RES_VERSION) ) {
        p_exit(hErrorDialog(E_GEN_FAIL, RS_BADRES)) ;
    }
    p_free(DatApp1) ;

    // This version will not run on S3 Classic, so put up an error message
    if ( (wserv_channel->conn.info.version_id & WS_TYPE_MASK) != WS_TYPE_S3A ) {
        p_exit(hErrorDialog(E_GEN_NSUP, RS_S3AONLY)) ;
    }

    // Load the name of the environment variable used to store preferences
    hLoadResource(RS_ENV, &DatApp1) ;

    p_bfil(&options, sizeof(options), 0) ;
    options.winpos = options.show = options.sound = 1 ;
    p_getenviron(DatApp1, p_slen(DatApp1), &options) ;

    self->wserv.help_index_id = RH_MAIN ;

    // Create engine
    soleng = f_newsend(CAT_SOLIPEG_SOLIPEG, C_SOLENG, O_ENG_INIT) ;
    p_send3(self->wserv.com, O_SCM_ENGINE, soleng) ;

    // Create window to be the main client window connected to the server
    //  Here f_new() creates a C_SOLBWIN object from the category SOLIPEG
    self->wserv.cli = f_newsend(    CAT_SOLIPEG_SOLIPEG,
                                    C_SOLBWIN,
                                    O_WN_INIT ) ;

    // Send the command manager NEW_GAME message
    p_send2(self->wserv.com, O_SCM_DO_NEW) ;

    // Make a Status bar visible
    wsEnable() ;
}

