// UNDOMAN.C - Undo manager

// This file includes methods for:

// C_UNDOMAN: the undo manager

#include <plib.h>
#include <solipeg.g>
#include <solipeg.rsg>
#include <hwim.h>



#pragma save, METHOD_CALL



#undef this
#define this    (self->undoman)

//  Macro:      TASK_PTR(index)
//  Returns:    &TASK[index] (A PR_TASK * in VAFLAT)
#define TASK_PTR(index) (*(PR_TASK **)p_send3(this.pList, O_VA_PREC, index))


// Initaliaze the undoman object, using category and class specified for
// the list manager
METHOD VOID undoman_uman_init(  PR_UNDOMAN *    self,
                                VOID *          pEngine,
                                INT             iCat,
                                INT             iClass ) {
    this.pEngine = pEngine ;
    this.pList = f_newsend( iCat,
                            iClass,
                            O_VA_INIT,
                            sizeof(PR_TASK *),
                            10 ) ;
    this.UndoIndex = 0 ;
}

// Submit a task - the task is sent a DO message
METHOD VOID undoman_uman_submit(    PR_UNDOMAN *    self,
                                    PR_TASK *       pTask ) {
    INT         nCount ;

    nCount = p_send2(this.pList, O_VA_COUNT) ;

    while ( this.UndoIndex < nCount ) {
        nCount-- ;
        p_send2(TASK_PTR(nCount), O_DESTROY) ;
        p_send3(this.pList, O_VA_DELETE, nCount) ;
    }

    p_send3(this.pList, O_VA_APPEND, &pTask) ;
    this.UndoIndex++ ;
    p_send3(pTask, O_TASK_DO, this.pEngine) ;
}

// Undo from the current position
METHOD VOID undoman_uman_undo(PR_UNDOMAN *self) {
    if ( p_send2(self, O_UMAN_CANUNDO) ) {
        this.UndoIndex-- ;
        p_send3(TASK_PTR(this.UndoIndex), O_TASK_UNDO, this.pEngine) ;
    }
}

// Redo from the current position
METHOD VOID undoman_uman_redo(PR_UNDOMAN *self) {
    if ( p_send2(self, O_UMAN_CANREDO) ) {
        p_send3(TASK_PTR(this.UndoIndex), O_TASK_DO, this.pEngine) ;
        this.UndoIndex++ ;
    }
}

// Is Redo allowed?
METHOD INT undoman_uman_canredo(PR_UNDOMAN *self) {
    return( this.UndoIndex < p_send2(this.pList, O_VA_COUNT) ) ;
}

// Is Undo allowed?
METHOD INT undoman_uman_canundo(PR_UNDOMAN *self) {
    return( this.UndoIndex > 0 ) ;
}

// Get text describing the Redo operation
METHOD VOID undoman_uman_redotxt(PR_UNDOMAN *self, TEXT *str) {
    if ( p_send2(self, O_UMAN_CANREDO) ) {
        p_send3(TASK_PTR(this.UndoIndex), O_TASK_TEXT, str) ;
    }
    else {
        *str = 0 ;
    }
}

// Get text describing the Undo operation
METHOD VOID undoman_uman_undotxt(PR_UNDOMAN *self, TEXT *str) {
    if ( p_send2(self, O_UMAN_CANUNDO) ) {
        p_send3(TASK_PTR(this.UndoIndex - 1), O_TASK_TEXT, str) ;
    }
    else {
        *str = 0 ;
    }
}

// Clear the undo history
// Fixed for 2.1
METHOD VOID undoman_uman_clear(PR_UNDOMAN *self) {
    this.UndoIndex = p_send2(this.pList, O_VA_COUNT) ;

    while ( this.UndoIndex > 0 ) {
        this.UndoIndex-- ;
        p_send2(TASK_PTR(this.UndoIndex), O_DESTROY) ;
    }
    p_send2(this.pList, O_VA_RESET) ;
}

