// WAVCONV.CPP
// Wav <-> Wve conversion dll
// Written by JP, Jan 1995
// (C) Copyright Psion PLC

// Version 1.00
                   
#include <stdlib.h>                 
#include <windows.h>
#include <mmsystem.h>
#include <string.h>
#include "WavConv.h"   
#include <psiwinxl.h>     

#define STEPSIZE 1000L
#define SIZE_BUF 0x1000L   
#define DELTA 0x20L              
#define SIZE_ERRBUF 0x20L
                                          
static short WErr=WErrUndefined;
static HINSTANCE hInst;  

static char inBuf[SIZE_BUF+DELTA];
static char outBuf[SIZE_BUF+DELTA];
static short bytesPerSample;
static short Amplification=1;                                           
static long inDataLength;
static long sampleRate;
                    
void leave(int panicNo)
{// Debug only, should never get called.
	panicNo=panicNo;
}

int WVEreadHeader(int sh)
{
    _lread(sh,&inBuf[0],16);
    if (memcmp(&inBuf[0],"ALawSoundFile**",16))
		{
		WErr=WErrBadWVEformat;
        return(FALSE);
 		}
    _lread(sh,&inBuf[0],2); // Garbage
    _lread(sh,&inDataLength,4);
    _lread(sh,&inBuf[0],10);// Garbage
	return(TRUE);
}

int WVEwriteHeader(int rh,long totOutLen)
{               
	int ret=TRUE;           
	SndFile head;
	strcpy(&head.Signature[0],ALawSignature);
	head.Version=0x100F;
	head.Samples=totOutLen;
	head.SilenceInTicks=0;
	head.Repeats=0;        
	memset(&inBuf[0],0x55,6);
	if (SignatureSize!=_lwrite(rh,(char*)&head.Signature[0],SignatureSize) ||
		sizeof(head.Version)!=_lwrite(rh,(char*)&head.Version,sizeof(head.Version)) ||
		sizeof(head.Samples)!=_lwrite(rh,(char*)&head.Samples,sizeof(head.Samples)) ||
		sizeof(head.SilenceInTicks)!=_lwrite(rh,(char*)&head.SilenceInTicks,sizeof(head.SilenceInTicks)) ||
		sizeof(head.Repeats)!=_lwrite(rh,(char*)&head.Repeats,sizeof(head.Repeats)) ||
		_lwrite(rh,&inBuf[0],6)!=6)
		{
		WErr=WErrWritingWVEheader;
		ret=FALSE;
		}
	return(ret);
}

int WAVreadHeader(int sh)
{
	short i;
	long totLen,locLen,pos,opos,j;
	_lread(sh,&inBuf[0],4);
	if (memcmp(&inBuf[0],"RIFF",4)) 
		{
		WErr=WErrBadWAVformat;
		return(FALSE);
		}
	_lread(sh,&totLen,4);
	_lread(sh,&inBuf[0],4);
	if (memcmp(&inBuf[0],"WAVE",4)) 
		{
		WErr=WErrBadWAVformat;
		return(FALSE);
		}
	for(;;)
		{
		_lread(sh,&inBuf[0],4);
		if(!memcmp(&inBuf[0],"fmt ",4))
			break;
		_lread(sh,&pos,4);
		totLen=totLen-8-pos;
		opos=_llseek(sh,0L,SEEK_CUR);
		opos+=pos;
		pos=_llseek(sh,pos,SEEK_CUR);
		if (pos<0 || pos!=opos)
			{
			WErr=WErrBadWAVformat;
			return(FALSE);
			}
		}
	_lread(sh,&locLen,4);
	_lread(sh,&i,2);
	if (i!=1)
		{
		WErr=WErrUnexpectedFormatTag; 
		return(FALSE);
		}
	_lread(sh,&i,2);
	if (i!=1)
		{
		WErr=WErrUnexpectedChannel;
		return(FALSE);
		}
	_lread(sh,&sampleRate,4);
	_lread(sh,&j,4);
	_lread(sh,&bytesPerSample,2);
	if (bytesPerSample!=1 && bytesPerSample!=2)
		{
		WErr=WErrUnexpectedBytesPerSample;
		return(FALSE);
		}
	if (j!=(long)bytesPerSample*sampleRate)
		{
		WErr=WErrStrangeSampleRates;
		return(FALSE);
		}
	_lread(sh,&i,2);
	if (i!=8*bytesPerSample)
		{
		WErr=WErrUnexpectedDataSize;
		return(FALSE);
		}
	locLen-=0x10;
	if (locLen)
		_lread(sh,&inBuf[0],(int)locLen);
	_lread(sh,&inBuf[0],4);
	if (memcmp(&inBuf[0],"data",4))
		{
		WErr=WErrBadWAVformat;
		return(FALSE);
		}
	_lread(sh,&inDataLength,4);
	if ((totLen-locLen-0x24)!=((inDataLength+1)&0xffffffeL)) // Cool files fail here but are ok.
		{
		WErr=WErrIncorrectHeaderLength;
//		return(FALSE);
		}
	return(TRUE);
}

int WAVwriteHeader(int rh, long totOutLen)
{
	int ret=TRUE;
    WAVTemplate wavFile;
    memcpy(&wavFile.Signature[0],"RIFF",4);
    memcpy(&wavFile.Format[0],"WAVEfmt ",8);
    wavFile.HeaderLength=16L;
    wavFile.FormatTag=1;
    wavFile.ChannelNo=1;
    wavFile.SampleRate=WAVSAMPLERATE;
    wavFile.BytesXSample=(long)(WAVSAMPLERATE*bytesPerSample);
    wavFile.BytesPerSample=bytesPerSample;   
    wavFile.EightXBytes=(short)(8*bytesPerSample);      
    memcpy(&wavFile.DataMark[0],"data",4);
    wavFile.DataLength=totOutLen*bytesPerSample; 
    wavFile.TotalFileLength=(totOutLen*bytesPerSample +36L);
    if (_lwrite(rh,(char*)&wavFile.Signature[0],4)!=4 ||
    	_lwrite(rh,(char*)&wavFile.TotalFileLength,sizeof(wavFile.TotalFileLength))!=sizeof(wavFile.TotalFileLength) ||
		_lwrite(rh,(char*)&wavFile.Format[0],8)!=8 ||
    	_lwrite(rh,(char*)&wavFile.HeaderLength,sizeof(wavFile.HeaderLength))!=sizeof(wavFile.HeaderLength) || 
    	_lwrite(rh,(char*)&wavFile.FormatTag,sizeof(wavFile.FormatTag))!=sizeof(wavFile.FormatTag) ||
    	_lwrite(rh,(char*)&wavFile.ChannelNo,sizeof(wavFile.ChannelNo))!=sizeof(wavFile.ChannelNo) ||
    	_lwrite(rh,(char*)&wavFile.SampleRate,sizeof(wavFile.SampleRate))!=sizeof(wavFile.SampleRate) ||
    	_lwrite(rh,(char*)&wavFile.BytesXSample,sizeof(wavFile.BytesXSample))!=sizeof(wavFile.BytesXSample) ||
    	_lwrite(rh,(char*)&wavFile.BytesPerSample,sizeof(wavFile.BytesPerSample))!=sizeof(wavFile.BytesPerSample) ||
    	_lwrite(rh,(char*)&wavFile.EightXBytes,sizeof(wavFile.EightXBytes))!=sizeof(wavFile.EightXBytes) ||
    	_lwrite(rh,(char*)&wavFile.DataMark[0],4)!=4 ||
    	_lwrite(rh,(char*)&wavFile.DataLength,sizeof(wavFile.DataLength))!=sizeof(wavFile.DataLength))
    	{
    	WErr=WErrWritingWAVheader;
		ret=FALSE;
		}
	return(ret);
}

int WVEreadData(int aSourceFile, long dataLen, long offset)
//
// Read dataLen bytes of 8bit data, ALaw expand to 13bit magnitude 
// and convert to signed short. Store in inBuf.
//
    {   
    int ret=TRUE;
    short* p=(short*)&inBuf[offset];
    unsigned char* q=(unsigned char*)&outBuf[0];

	if ((offset+dataLen)>=(SIZE_BUF+DELTA))
		leave(WPanicBufferOverflow);           
	if (_lread(aSourceFile,&outBuf[0],(unsigned int)dataLen)!=(unsigned int)dataLen)
		{
		WErr=WErrReadingFile;
		ret=FALSE;
		goto Leave;
		} 
	int i;
    for(i=0;i<dataLen;i++)
        {
	    int s, y, x;
        x=(int)*q++;
        x=(x^XORMASK)&MASK8;
        s=(x&MASK3)>>4;
        y=0;
        if (s)
            {
            y=0x10;
            s-=1;
            }
        y=(((y+(x&MASK4)) << 1)+1) << s;
        if (x&BIT8)
            y= -y;
        y+=0x1000;
        if (y>=0x1800 || y<=0x800) /* Should never occur */
            leave(WPanicCalculationOverflow);
		if (bytesPerSample==2)
			{
			y<<=2;
	       	y-=0x4000;
	       	*p++=(short)(y<<1);
			}
		else
			*p++=(short)(y>>5);
        }
Leave:
    return(ret);
    }
                              
int WAVreadData(int aSourceFile,long i,long offset)
//
// Read i bytes from aSourceFile and put them into
// inBuf starting from location offset.
//
{	                        
	int ret=TRUE;
	char* pIn=&inBuf[offset];
	char* pEnd=pIn+i;
	
	if ((offset+i)>=(SIZE_BUF+DELTA))
		leave(WPanicBufferOverflow);           
	if (_lread(aSourceFile,&inBuf[offset],(unsigned int)i)!=(unsigned int)i)
		{
		WErr=WErrReadingFile;
		ret=FALSE;
		goto Leave;
		}
	if (bytesPerSample==1)
		while (pIn<pEnd)
			*pIn++=(char)((*pIn)-0x80);
Leave:
	return(ret);
}
                            
int WriteData(int aResultFile,long outLen)
{                         
	int ret=TRUE;
	if (outLen!=(long)_lwrite(aResultFile,&outBuf[0],(int)outLen))
		{
		WErr=WErrWritingData;
		ret=FALSE;
		}
	return(ret);
}

                    
char aCompand(short x)
//
// Compress input 16-bit 2's complement integer (13-bit magnitude) to
// 8-bit signed compressed number using A-law.
//
{
	int p=BIT8, s, y;
    if (x&0x1000)
		{
		x= (short)(-x);
		x&=0xFFF;
		p=0;
		}
    if (x&(MASK4<<8))
    	{
		if (x&(MASK2<<10))
		    s=(x&(ONE<<11)) ? 7 : 6;
		else
		    s=(x&(ONE<<9)) ? 5 : 4;
		}
    else
		{
		if (x&(MASK2<<6))
		    s=(x&(ONE<<7)) ? 3 : 2;
		else
		    s=(x&(ONE<<5)) ? 1 : 0;
		}
    if (s==0)
		y=x>>1;
    else
		y=((x>>s)&MASK4)|(s<<4);
    return((char)((~((y|p)^MAGICXOR))&MASK8));
}

long WVE2wav8(long inData, long* offset)
//
// Convert WVE data in inBuf to WAV data in outBuf
//
{
	static long d=0;
	static const float con=((float)WAVSAMPLERATE/(float)WVESAMPLERATE)*(float)STEPSIZE;
	short* m;
	short* p;
	unsigned char* q;
	short x;  
    
    q=(unsigned char*)&outBuf[0];
    m=(short*)&inBuf[0];
    while(q<(unsigned char*)&outBuf[SIZE_BUF])
       	{
        x=(short)((*(m+1)-*m)*(float)d/con);
        if ( (x+*m)<(-1*0x100) || (x+*m)>0x100)
        	leave(WPanicCalculationOverflow);
        *q++=(unsigned char)(x+*m);
        d+=STEPSIZE;
        if(d>=con)
            {
            d-=(long)con;
            m++;
            if ((m+1)>=(short*)&inBuf[inData])
            	break;
            }
        }
    p=(short*)&inBuf[0];
    *offset=&inBuf[inData]-(char*)m;
    while(m<(short*)&inBuf[inData])
    	*p++=*m++;
    return((long)((char*)q-&outBuf[0]));
}	

long WVE2wav16(long inData, long* offset)
//
// Convert WVE data in inBuf to WAV data in outBuf
//
{
	static long d=0;
	static const float con=((float)WAVSAMPLERATE/(float)WVESAMPLERATE)*(float)STEPSIZE;
	short* m;
	short* p;
	short* q;
	short x;  
    
    q=(short*)&outBuf[0];
    m=(short*)&inBuf[0];
    while(q<(short*)&outBuf[SIZE_BUF])
       	{
        x=(short)((*(m+1)-*m)*(float)d/con);
        if ( (x+*m)<(-1*0x5000) || (x+*m)>0x5000)
        	leave(WPanicCalculationOverflow);
        *q++=(short)(x+*m);
        d+=STEPSIZE;
        if(d>=con)
            {
            d-=(long)con;
            m++;
            if ((m+1)>=(short*)&inBuf[inData])
            	break;
            }
        }
    p=(short*)&inBuf[0];
    *offset=&inBuf[inData]-(char*)m;
    while(m<(short*)&inBuf[inData])
    	*p++=*m++;
    return((long)((char*)q-&outBuf[0]));
}	
                            
int WAV2wve8(long inData, long* offset)
// 
// Convert WAV data in inBuf to WVE data in outBuf.
// Recycle any unused bytes into inBuf.
// Return number of bytes written to outBuf.
//
{   
	static long d=0;
	long con=(sampleRate>>3)*(STEPSIZE/1000L);
	long z=sampleRate/8000L+1;
	long r,x,val;
	char* q=&outBuf[0];
	char* pIn=&inBuf[0];           
	char* pEnd=pIn+inData+*offset;                    
	
	while (pIn+z<pEnd)
		{             
		d=STEPSIZE-d;
		val=d*(*pIn++);
		d=con-d;
		x=d/STEPSIZE;
		d-=STEPSIZE*x;
		for (int j=0;j<x;j++)
			val+=STEPSIZE*(*pIn++);
		val+=d*(*pIn);       
		val*=16L;
		val/=con;
		if (val>2048||val<-2048)
			leave(WPanicCalculationOverflow);
		*q++=aCompand((short)val);
		}                    
	r=(long)(pEnd-pIn);
	memcpy(&inBuf[0],pIn,(int)r);
	*offset=r;
	return(q-&outBuf[0]);
}
                              
int WAV2wve16(long inData, long* offset)
// 
// Convert WAV data in inBuf to WVE data in outBuf.
// Recycle any unused bytes into inBuf.
// Return number of bytes written to outBuf.
//
{   
	static long d=0;
	long con=(sampleRate>>3)*(STEPSIZE/1000L);
	long z=sampleRate/8000L+1;
	long r,x,val;
	char* q=&outBuf[0];
	short* pIn=(short*)&inBuf[0];           
	short* pEnd=pIn+((inData+*offset)>>1);                    
	
	while (pIn+z<pEnd)
		{             
		d=STEPSIZE-d;
		val=d*(*pIn++);
		d=con-d;
		x=d/STEPSIZE;
		d-=STEPSIZE*x;
		for (int j=0;j<x;j++)
			val+=STEPSIZE*(*pIn++);
		val+=d*(*pIn);   
		if (pIn>=pEnd)
			leave(-1);    
		val/=16L;
		val/=con;
	if (val>2048||val<-2048)
			leave(WPanicCalculationOverflow);
		*q++=aCompand((short)val);
		}                    
	r=(long)(pEnd-pIn)<<1;
	memcpy(&inBuf[0],pIn,(int)r);
	*offset=r;
	return(q-&outBuf[0]);
}

int GenerateWVEfile(int sh,int rh, XMSTATPROC statRtn)
{             
	int ret=WInfConvIncomplete;
	long inLen,outLen,totOutLen,i;
	long offset=0;

	if (!WAVreadHeader(sh)) goto Leave;
	inLen=inDataLength;
	totOutLen=(long)((double)(inLen*8000.0)/sampleRate/(long)bytesPerSample);
	i=(totOutLen>>10)+1;
	totOutLen=i<<10;
	if (!WVEwriteHeader(rh,totOutLen)) goto Leave;
	while (inLen)
		{                   
		i=(inLen>SIZE_BUF ? SIZE_BUF : inLen);
		inLen-=i;	
		if (!WAVreadData(sh,i,offset)) goto Leave;
		if (bytesPerSample==1)
			outLen=WAV2wve8(i,&offset);
		else
			outLen=WAV2wve16(i,&offset);
		if (!WriteData(rh,outLen)) goto Leave;                         
		totOutLen-=outLen;
		if (totOutLen<=0) leave(WPanicTooMuchOutput);
		if ((*statRtn)((int)(((inDataLength-inLen)*100L)/inDataLength)))
			{
			WErr=WErrStatusReturnTrue;
			goto Leave;
			}
		}         
	if (totOutLen>SIZE_BUF) leave(WPanicSilenceAtEndOverflow);
	if (totOutLen>0)
		{
		memset(&outBuf[0],0x55,(int)totOutLen);
		WriteData(rh,totOutLen);
		}	
	ret=WInfConvCompleted;    
Leave:
	return(ret);
}                                

int GenerateWAVfile(int sh,int rh, XMSTATPROC statRtn)
{
	int ret=WInfConvIncomplete;
	long inLen,outLen,totOutLen,i; 
	long offset=0;
		                     
	if (!WVEreadHeader(sh))
		goto Leave;
	inLen=inDataLength;
    totOutLen=(long)((inLen*WAVSAMPLERATE)/WVESAMPLERATE);
    totOutLen=((totOutLen>>10)+1)<<10;
 	if (!WAVwriteHeader(rh,totOutLen))
		goto Leave;
	while (inLen+offset)
		{                   
		i=(inLen>(SIZE_BUF/2-offset/2) ? (SIZE_BUF/2-offset/2) : inLen);
		inLen-=i;	
		if (!WVEreadData(sh,i,offset))
			goto Leave;
		if (bytesPerSample==1)
			outLen=WVE2wav8(2*i+offset,&offset);
		else
			outLen=WVE2wav16(2*i+offset,&offset);
		if (!WriteData(rh,outLen))
			goto Leave;                         
		totOutLen-=outLen/bytesPerSample;
		if ((*statRtn)((int)(((inDataLength-inLen)*100L)/inDataLength)))
			goto Leave;
		}         
	ret=WInfConvCompleted;
	if (totOutLen>(SIZE_BUF/bytesPerSample)) leave(WPanicSilenceAtEndOverflow);
	for(i=0;i<totOutLen;i++)
		if (bytesPerSample==2)
			*(short*)&outBuf[bytesPerSample*i]=0x0010;
		else
			outBuf[i]=(unsigned char)0x80;
	WriteData(rh,bytesPerSample*totOutLen);
Leave:
	return(ret);		
}
                               
int WAVcheckHeader(int fh)
// Check file is WAV format
{                
	int ret=2;		
	_lread(fh,&inBuf[0],4);
	if (memcmp(&inBuf[0],"RIFF",4)) 
		ret=0;
	_lread(fh,&inBuf[0],8);
	if (memcmp(&inBuf[4],"WAVE",4)) 
		ret=0;
	return(ret);
}
                           
int WVEcheckHeader(int fh)
// Check file is WVE format
{             
	int ret=1;
	
	_lread(fh,&inBuf[0],15);
	if (memcmp(&inBuf[0],"ALawSoundFile**",15))
		ret=0;		         
	return(ret);
}				   

void ExamineHardwareConfig(void)
// Look for a 16bit soundblaster, otherwise assume 8bit
// conversion and set bytesPerSample accordingly.
{
	WAVEOUTCAPS wOutCaps;
	bytesPerSample=1;
	unsigned int numDev=waveOutGetNumDevs();
	while(numDev--)
		{
		waveOutGetDevCaps(numDev,&wOutCaps,sizeof(WAVEOUTCAPS));
		if (wOutCaps.dwFormats & (SIXTEEN_BIT))
			bytesPerSample=2;
		}
}

EXPORT_C int WINAPI _export XLSelectApp(LPSTR aFileName)
//
// Check that file aFileName is a valid WAV file.
//
{ 
	int ret=0;
	int fh=_lopen(aFileName, READ | OF_SHARE_COMPAT);
	if (fh==HFILE_ERROR) 
		WErr=WErrOpeningWAVfile;
	else
		ret=WAVcheckHeader(fh);
	_lclose(fh);
	return(ret);
}

EXPORT_C int WINAPI _export XLSelectFormat(LPSTR aFileName)
//
// Check that file aFileName is a vaild WVE file.
//
{
	int ret=0;
	int fh = _lopen(aFileName, READ | OF_SHARE_COMPAT);
	if (fh==HFILE_ERROR) 
		WErr=WErrOpeningWVEfile;
	else 
		ret=WVEcheckHeader(fh);
	_lclose(fh);
	return(ret);
}

EXPORT_C BOOL WINAPI _export XLImport(XMINFO FAR* info, LPSTR aSourceFile,
					 LPSTR aResultFile, unsigned short xferMode,
					 XMSTATPROC statusRtn)
//  Translate Windows aSourceFile into Psion format aResultFile.
//  LPXMINFO info; 			Context information useful for the translator
//	int xferMode;  			1 Overwrite, 2 Append, 3 Merge/Update
//	XMSTATPROC statusRtn;	Ptr to percentage completed function
{	
	info=info;
	BOOL ret=FALSE;
	int sh = _lopen(aSourceFile, READ | OF_SHARE_COMPAT);
	int rh = NULL;
	if (sh==HFILE_ERROR)
		WErr=WErrOpeningWAVfile;
	else if (xferMode!=1)
		WErr=WErrModeNotSupported;
	else if ((rh=_lcreat(aResultFile,0))==HFILE_ERROR)
		WErr=WErrCreatingFile;
	else if (_lclose(rh),(rh=_lopen(aResultFile, READ_WRITE | OF_SHARE_COMPAT))==HFILE_ERROR)
		WErr=WErrOpeningWVEfile;
	else if (!statusRtn)
		WErr=WErrNoStatusRtn;
	else if (GenerateWVEfile(sh,rh,statusRtn)==WInfConvCompleted)
			ret=TRUE;
	_lclose(sh);
	_lclose(rh);
	return(ret);
}

EXPORT_C BOOL WINAPI _export XLExport(XMINFO FAR* info, LPSTR aSourceFile,
					 LPSTR aResultFile, unsigned short xferMode,
					 XMSTATPROC statusRtn)
//  Translate Psion format aSourceFile into Windows aResultFile.
//  LPXMINFO info; 			Context information useful for the translator
//	int xferMode;  			1 Overwrite, 2 Append, 3 Merge/Update
//	XMSTATPROC statusRtn;	Ptr to percentage completed function
{           
	info=info;
	BOOL ret=FALSE;
	int sh = _lopen(aSourceFile, READ | OF_SHARE_COMPAT);
	int rh = NULL;
	ExamineHardwareConfig();
	if (sh==HFILE_ERROR)
		WErr=WErrOpeningWVEfile;
	else if (xferMode!=1)
		WErr=WErrModeNotSupported;
	else if ((rh=_lcreat(aResultFile,0))==HFILE_ERROR)
		WErr=WErrCreatingFile;
	else if (_lclose(rh),(rh=_lopen(aResultFile, READ_WRITE | OF_SHARE_COMPAT))==HFILE_ERROR)
		WErr=WErrOpeningWAVfile;
	else if (!statusRtn)
		WErr=WErrNoStatusRtn;
	else if (GenerateWAVfile(sh,rh,statusRtn)==WInfConvCompleted)
			ret=TRUE;
	_lclose(sh);
	_lclose(rh);
	return(ret);
}

EXPORT_C int WINAPI _export XLGetLastErrorMsg(LPSTR msg, int nMsg)
//
// Copy last error message into buffer msg + trailing /0
// nMsg = maxlength of buffer
// 
{
	int n;
	char dum[SIZE_ERRBUF];
		
	n=LoadString(hInst,WErr,dum,SIZE_ERRBUF);
	dum[n]='\0';
	lstrcpyn(msg,(LPCSTR)&dum[0],nMsg);
	msg[nMsg-1]='\0';
	return(lstrlen(msg));
}


EXPORT_C int WINAPI _export LibMain(HANDLE hInstance, unsigned short wDataSeg,
					unsigned short cbHeapSize, LPSTR lpCmdLine)
{                       
	wDataSeg=wDataSeg;
	lpCmdLine=lpCmdLine;
	if (cbHeapSize > 0)
#if defined(__VC32__)
	;
#else
		UnlockData(0);
#endif
	hInst = hInstance;
	return(TRUE);
}  

EXPORT_C WORD WINAPI XLInstall(LPSTR lppspd)
{
	return *(WORD*) lppspd;
}
