/*
   WRD2TXT.C

   Started 9/5/94
   Started by Tim, extended by AndrewB
   Function: Converts a .WRD file to ASCII text
*/

#include <p_std.h>
#include <p_file.h>
#include <p_sys.h>

#define SIGLEN 16                      /* Byte length of signature */

enum {	PARAGRAPH_END = 0,
		HARD_HYPHEN = 7,
		FORCED_BREAK = 10,
		SOFT_HYPHEN = 14,
		HARD_SPACE = 15,
		SPACE = 32
		};

LOCAL_D VOID *pInFcb=NULL;            /* Word processor binary file pointer */
LOCAL_D VOID *pOutFcb=NULL;          /* ASCII output file pointer */
LOCAL_D TEXT signature[SIGLEN]="PSIONWPDATAFILE"; /* .WRD file signature */
LOCAL_D TEXT inBuf[P_FBLKSIZE];			/* Temporary input buffer */
LOCAL_D TEXT outBuf[P_FBLKSIZE*2];		/* Temporary output buffer */

LOCAL_D INT Mode;
LOCAL_D UINT Lines=1;
LOCAL_D UINT Tabs;
LOCAL_D ULONG ByteCount=0;                
GLREF_C VOID p_getcom(TEXT *buf);      /* Prototype for comand line function */

LOCAL_C VOID exit(VOID)
	{
	p_close(pInFcb);
	p_close(pOutFcb);
	p_exit(TRUE);
	}

LOCAL_C VOID Panic(INT err, TEXT *msg)
/* General purpose panic reporter */
	{
	TEXT buf[60];

	p_puts(msg);
	if (err)
		{
		p_errs(&buf[0],err);
		p_puts(&buf[0]);
		}
	exit();
	}

LOCAL_C VOID Seek(INT sense,INT seekpos)
/* Look for a place in the .WRD file */
	{
	INT err;
	LONG lint;

	lint=seekpos;
	err=p_seek(pInFcb,sense,&lint);
	if (err<0)
		{
		Panic(err,"Error seeking in input file");
		}
	}

LOCAL_C VOID Read(VOID *pb,INT len)
/* Try to read len bytes from .WRD file,
Panic if an error occurs or there aren't enough bytes left */
	{
	INT val;

	val=p_read(pInFcb,pb,len);
	if (val<len)
		{
		Panic(val>0? 0: val,"Error reading input file");
		}
	}

LOCAL_C VOID Write(VOID *pb,INT len)
	{
	INT err;

	err=p_write(pOutFcb,pb,len);
	if (err<0)
		{
		Panic(err,"Error writing output file");
		}
	}

LOCAL_C VOID CheckSignature(VOID)
/* Check that the input file is a PSION Word Processor file */
	{
	TEXT teststring[SIGLEN];
	INT err;

	Read(&teststring[0],SIGLEN);
	err=p_bcmp(&teststring[0],SIGLEN,&signature[0],SIGLEN);
	if (err)  /* Only returns zero if strings matched */
		{
		Panic(0,"Error, not a valid Psion WRD file");
		}
	}

LOCAL_C VOID ProcessBlockNoWrap(INT Nbytes,INT *pCurline)
/* This function outputs the text in paragraph or no-wrapped line mode */
	{
	TEXT *pIn;
	TEXT *pOut;
	INT CharCode;
	INT RepTab;

	pIn=(&inBuf[0]);
	pOut=(&outBuf[0]);
	RepTab=0;
	Read(pIn,Nbytes);
	
	while (Nbytes--)
		{
		if ((Mode!=0)&&(*pCurline==-Mode))
			{
			*pCurline=0;
			*pOut++='\r';
			*pOut++='\n';
			Lines++;
			}
		CharCode=(*pIn);

		switch (CharCode)
			{
		case PARAGRAPH_END:
		case FORCED_BREAK:
			*pOut++='\r';
			CharCode='\n';
			*pCurline=-1;
			Lines++;
			break;
		case HARD_HYPHEN:
			CharCode='-';
			break;
		case SOFT_HYPHEN:
			continue;
		case HARD_SPACE:
			CharCode=' ';
			break;
		case '\t':
			if ((Tabs!=0)&&(RepTab==0))
				{
				RepTab=Tabs-1;
				CharCode=' ';
				}
			else if (Tabs!=0)
				{
				RepTab--;
				Nbytes++;
				CharCode=' ';
				}
			break;
			}

		if (RepTab==0)
			{
			pIn++;
			}

		*pOut++=CharCode;
		(*pCurline)++;
		}

	Write(&outBuf[0],pOut-&outBuf[0]);

	ByteCount+=(pOut-&outBuf[0]);
    }

#define NEW_PARAGRAPH 0				
#define EOL_WITH_BREAK_POINT 1
#define EOL_WITH_NO_BREAK_POINT 2
#define END_OF_FILE 3
#define NEW_LINE 4

LOCAL_C INT ProcessBlockWordWrap(UINT Nbytes,INT *pCurline)
/* This function outputs the text in line mode with word wrapping */
	{
	TEXT *pIn;
	TEXT *pEndbuf; /* A pointer to the end of the current buffer */
	TEXT *pOut;
	TEXT *pBreakPoint; /* A ponter to the last suitable break point */
	TEXT *pLastSpace; /* A pointer to the last space */
 	INT CharCode;
	INT Btype=0;
	INT TempLoc=0;
	INT LastBreak=0;
	INT RepTab; /* Temp counter to add tabs */
	INT XtraSpace;	 /* Count the extra spaces from expanded tabs */

	pIn=(&inBuf[0]);
	pEndbuf=pIn+Nbytes-1;
	pOut=(&outBuf[0]);
	pBreakPoint=pIn;

/* First, read in the buffer */	
	Read(pIn,Nbytes);

	while (pIn<=pEndbuf)
		{

/* Do we have enough information to process properly?
   i.e. does our current buffer extend beyond the eol?
	If not, write what we have and re-read it */
		if ((pEndbuf-pIn)<=(Mode-*pCurline))
			{
			if (Nbytes==P_FBLKSIZE)
				{	/* Unless we are at the end of the file... */
				Write(&outBuf[0],pOut-&outBuf[0]);
				ByteCount=ByteCount+(pOut-&outBuf[0]);
				return(LastBreak-Nbytes); /* how much we seek back */
				}
			}

/* Pass 1 - find out where to break the line.
   Either we break at the first forced break or paragraph end, or...
   we break at the last space before the end of the line */

		Btype=0;
		pLastSpace=0;
		RepTab=0;
		XtraSpace=0;

		FOREVER
			{
			CharCode=*pBreakPoint;

			switch (CharCode)
				{
			case PARAGRAPH_END:
			case FORCED_BREAK:
				goto pass2;
			case SOFT_HYPHEN:
			case SPACE:
				pLastSpace=pBreakPoint;
				break;
			case '\t':
				pLastSpace=pBreakPoint;
				if (RepTab==0)
					{
					RepTab=Tabs;
					}
				break;
				}

			if ((*pCurline+TempLoc-LastBreak+XtraSpace)>=Mode)
				{  /* Ah, we`re at the end of a line... */
				Btype=EOL_WITH_BREAK_POINT;

				if (!pLastSpace)
					{
					if (*pCurline==0) /* Oh dear, no suitable breaks */
						{
						Btype=EOL_WITH_NO_BREAK_POINT;
						p_printf("Warning, word not wrapped properly on line %d",Lines);									
						break;
						}

					/* eol, lets have a new line and write the buffer */
					*pOut++='\r';
					*pOut++='\n';
					TempLoc-=pBreakPoint-pIn;
					break;
					}
				else
					{
					TempLoc-=(pBreakPoint-pLastSpace);
					break;
					}
				}

			/* Ah, we`re at the end of the file,
			   Lets just dump the rest out */
			if (pBreakPoint>=pEndbuf)
				{
				Btype=END_OF_FILE;
				break;
				}


		    if (RepTab==1)
				{
				TempLoc--;
				}

			if (RepTab>0)
				{
				RepTab--;
				XtraSpace++;
				}

			if (RepTab==0)
				{
				pBreakPoint++;
				TempLoc++;
				}

			}

		pass2:

/* Bug catcher... */

	if (pBreakPoint<pIn)
		{
		p_puts("Internal Error, exiting");
		exit();
		}

/* Ok, now we know where we`re going to break...
   at pLastSpace for btype=1, or at pBreakPoint for btype=0,2,3

   Pass 2: Now we copy from the in buffer to the out buffer
   making the appropriate replacements.*/

		LastBreak=TempLoc;

		RepTab=0;

		if (Btype==EOL_WITH_BREAK_POINT)
			{
			pBreakPoint=pLastSpace;
			}

		while(pIn<=pBreakPoint)
			{
			CharCode=*pIn;

/* Do the replacements */

			switch (CharCode)
        		{
			case PARAGRAPH_END:
			case FORCED_BREAK:
           		*pOut++='\r';
           		CharCode='\n';
				(Lines)++;
				Btype=NEW_LINE;
				break;
			case HARD_HYPHEN:
           		CharCode='-';
				break;
       		case SOFT_HYPHEN:
           		CharCode='-';
				break;
       		case HARD_SPACE:
           		CharCode=32;
				break;
			case '\t':
				if (RepTab==0)
					{
					RepTab=Tabs;
					}
				CharCode=32;
				break;
				}

/* Last char, if EOL with break => overwrite with CR+LF
   OR, if EOL with no break => add CR+LF */
			if (pIn==pBreakPoint)
				{
				if (Btype==EOL_WITH_BREAK_POINT)
					{
					if (CharCode==32)
						{  /* overwrite */
						*pOut++='\r';
						*pOut++='\n';
						*pCurline=0;
						(Lines)++;
						Btype=NEW_LINE;
						}
					}
				else if (Btype==EOL_WITH_NO_BREAK_POINT)
					{
					*pOut++='\r';
					CharCode='\n';
					*pCurline=0;
					(Lines)++;
					}
				}

			if (RepTab!=0)
				{
				RepTab--;
				}

			if (RepTab==0)
				{
				pIn++;
				}

/* Delete any extra spaces at the beginning of a line */
			if ((*pCurline==0)&&(CharCode==32))
				{
				; 
				}
			else
				{
				*pOut++=CharCode;
				(*pCurline)++;
				}
			}

		if (Btype!=END_OF_FILE)
			{
			*pCurline=0;
			}

		if (Btype==EOL_WITH_BREAK_POINT)
			{
			pIn--;
			}

		if (Btype==NEW_LINE)
			{
			pBreakPoint++;
			TempLoc++;
			LastBreak++;
			}
		}

/* Write out the buffer... */

	Write(&outBuf[0],pOut-&outBuf[0]);

	ByteCount+=pOut-&outBuf[0];

	return(0);
    }

#define DATARECNUM 8                   /* Number of data record */
#define HEADER_SIZE 40

LOCAL_C VOID ExtractTextFromWrd(VOID)
/* Discard the extra information in a word file and extract the text */
	{
	struct
		{
		INT Type; /* Record type number */
		UINT Len; /* Record data length */
		} Record;
	UINT Nbytes;     /* Number of bytes in block */
	INT Curline=0;   /* For line mode, the chars on the current line */
	INT Back;

	Seek(P_FABS,HEADER_SIZE);            /* Skip header */
	/* Now look for text data record */
	FOREVER
		{
		Read(&Record,sizeof(Record));
		if (Record.Type==DATARECNUM)     /* Got to text record */
			{
			break;
			}
			/* Not got to text yet, so go on */
		Seek(P_FCUR,Record.Len); /* Skip record data */
        }

    /* In Mode=0 (paragraph) we can just read in and break at
      	the appropriate places...while, Mode<0 breaks at width Mode */
	if (Mode<1)
		{
		while (Record.Len)
			{
			Nbytes=(Record.Len>P_FBLKSIZE? P_FBLKSIZE: Record.Len);
			ProcessBlockNoWrap(Nbytes,&Curline);
			Record.Len-=Nbytes;
			}
         /* but....in line mode (max line width=mode) we need
			word-wrap....hmmmm..... */
		}
	else
		{
		while (Record.Len)
			{
			Nbytes=(Record.Len>P_FBLKSIZE? P_FBLKSIZE: Record.Len);
			Back=ProcessBlockWordWrap(Nbytes,&Curline);
			if (Back!=0)
				{
				Seek(P_FCUR,Back);
				}
			Record.Len-=(Nbytes+Back);

			}
		}
	}    

LOCAL_C VOID OpenFiles(TEXT *pInFilNam, TEXT *pOutFilNam)
/* Open .WRD and .TXT files */
	{    
	INT err;

	err=p_open(&pInFcb,&pInFilNam[0],P_FOPEN|P_FRANDOM); 
	if (err<0)
		{
		Panic(err,"Error opening Word Processor file");
		}

    /* Open .TXT file for writing */
	err=p_open(&pOutFcb,&pOutFilNam[0],P_FSTREAM_TEXT|P_FUPDATE|P_FREPLACE);
	if (err<0)
		{
		Panic(err,"Error opening text file");
		}
	}

LOCAL_C VOID DisplaySyntax(VOID)
	{
	p_puts("Syntax: WRD2TXT <input_file[.wrd]> [-O<output_file[.txt]>]");
	p_puts("                 [-M<mode>] [-T<tab_size>] [-Q]");
	p_puts("");
	p_puts("mode=0: Paragraph mode (default) ");
	p_puts("mode>0: Line mode with word wrap - lines broken at width <mode>");
	p_puts("mode<0: Line mode with no word wrap - lines broken at width <-mode>");
	p_puts("");
	p_puts("Paragraph mode:");
	p_puts("If -T option not supplied, tabs left in");
    p_puts("If -T supplied, tabs replaced by <tab_size> spaces");
	p_puts("");
	p_puts("Line mode:");
	p_puts("Tabs replaced by <tab_size> spaces, default 1");
	p_exit(TRUE);
	}

LOCAL_C VOID GetCommandLine(TEXT *pInFilNam, TEXT *pOutFilNam)
	{
	TEXT *argv[3];                 /* Text arrays for command arguments */
	P_GETFLG arg;                  /* Argument struct */
	P_GETFLG oarg;                 /* Output flag argument struct */
	INT err;
	TEXT combuf[P_CMDMAX]; /* buffer for flag dissection */
	TEXT *oo=NULL;          /* Flag buffer */

	Tabs=0;	
	Mode=0;				/* Default to mode 0, paragraph */

	p_puts("");
	p_puts("Word Processor to Text File Converter V1.02 (c) Psion Plc 1994");
	p_puts("");
																 
	arg.argc=0;
	arg.argm=4;
	arg.argp=&argv[0];
	oarg.argm=1;           /* Only one output flag allowed */
	oarg.argc=0;
	oarg.argp=&oo;          /* Point to text buffer */

	p_getcom(&combuf[0]);

	p_sconf(&combuf[0]);

	err=p_getflg(&combuf[0],"O*,M#,T#",&oarg,&Mode,&Tabs,&arg);

	if (err<0 || !arg.argc)
		{
		DisplaySyntax();
		}

	if (oarg.argc>1 || arg.argc>1)
		{
		Panic(0,"Too many command line arguments");
		}

	err=p_fparse(argv[0],".WRD",pInFilNam,NULL);

	if (err<0)
		{
		DisplaySyntax();
		}

	if (oarg.argc)
		{
		err=p_fparse(&oo[0],".TXT",pOutFilNam,NULL);
		if (err<0)
			{
			DisplaySyntax();
			}
		}
	else
		{
		err=p_fparse(".TXT",pInFilNam,pOutFilNam,NULL);
		if (err<0)
			{
			DisplaySyntax();
			}
		}

	err=p_scmp(pInFilNam,pOutFilNam);
	if (!err)
		{
		p_puts("Input and Output filenames identical!");
		p_puts("");
		DisplaySyntax();
		}

	if (Tabs>=512)
		{
		Tabs=511;
		}

	if (!Mode)
		{
		p_puts("Converting file in Paragraph mode");
		if (Tabs!=0)
			{
			p_printf("Replacing tabs with %d spaces",Tabs);
			}
		}
	else
		{
		if (Tabs==0)
			{
			Tabs=1; /* Default tab conversion for line mode */
			}
	
		if (Mode>0)
			{
			if (Mode>=P_FBLKSIZE)
				{
				Mode=P_FBLKSIZE-1;
				}
			p_printf("Converting file in Line mode, with word wrap, max line length %d",Mode);
			}

		if (Mode<0)
			{
			if (Mode<=-P_FBLKSIZE)
				{
				Mode=-P_FBLKSIZE+1;
				}
			p_printf("Converting file in Line mode, with no word wrap, max line length %d",-(Mode));
 			}
		}
	}

GLDEF_C VOID main(VOID)
	{
	TEXT InFilNam[P_FNAMESIZE];
	TEXT OutFilNam[P_FNAMESIZE];

	/* Set up filing system for DOS */
	GLREF_D P_DEVICE p_file;
	p_inst(&p_file,NULL);

	/* Check the command line options */
	GetCommandLine(&InFilNam[0],&OutFilNam[0]);

	/* Open the input and output files */
	OpenFiles(&InFilNam[0],&OutFilNam[0]);

	/* Is our input a valid Psion WRD file? */
	CheckSignature();

	/* Convert it to straight ASCII in the required way */ 
	ExtractTextFromWrd();

	/* Tell the user what happened */
	p_printf("Output file name: %s",OutFilNam);
	p_printf("Size of output file: %Lu bytes",ByteCount);
	p_printf("Number of lines in output file: %d",Lines);

	exit(); 
	}
